class _GLUECLAMP_:
    _imports_ = (
        '_parent.UniSet:IdentitySetMulti',
        '_parent.UniSet:IdentitySet',
        '_parent.View:_hiding_tag_',
        '_root.guppy:sets',
        '_root.guppy.sets:NodeSet',
        '_root.guppy.sets:ImmNodeSet',
        '_root.guppy.sets:MutNodeSet',
        '_root.guppy.sets:immbit',
        '_root.guppy.sets:immbitrange',
        '_root.guppy.sets:immbitset',
        '_root.guppy.sets:mutbitset',
    )

    def _get_emptynodeset(self):
        return self.immnodeset()

    def immnodeset(self, it=()):
        return self.sets.immnodeset(it, self._hiding_tag_)

    def immnodeset_union(self, sets):
        return self.sets.immnodeset_union(sets, self._hiding_tag_)

    def mutnodeset(self, *args, **kwds):
        s = self.sets.mutnodeset(*args, **kwds)
        s._hiding_tag_ = self._hiding_tag_
        return s



from guppy.heapy.test import support


class FirstCase(support.TestCase):
    def setUp(self):
        support.TestCase.setUp(self)
        self.OH = self.heapy.OutputHandling

    def test_1(self):
        class T:
            def __init__(self, test, numlines, get_num_lines=None, get_more_msg=None):
                mod = test.OH

                def get_line_iter():
                    for i in range(numlines):
                        yield '%d' % i

                self.mod = mod
                mod.setup_printing(
                    self,
                    get_line_iter=get_line_iter,
                    max_more_lines=4,
                    get_num_lines=get_num_lines,
                    get_more_msg=get_more_msg)

        self.aseq(str(T(self, 4)), '0\n1\n2\n3')

        t = T(self, 6, lambda: 6)

        self.aseq(
            str(t), "0\n1\n2\n3\n<Lines 0..3 of 6. Type e.g. '_.more' for more.>")
        x = t.more
        self.aseq(str(x), '4\n5')
        self.aseq(
            str(x.top), "0\n1\n2\n3\n<Lines 0..3 of 6. Type e.g. '_.more' for more.>")

        self.aseq(str(x.all), "0\n1\n2\n3\n4\n5")

        t = T(self, 6, get_more_msg=lambda f, t: '<%d more rows>' % (6-t))
        self.aseq(str(t), '0\n1\n2\n3\n<3 more rows>')


def test_main(debug=0):
    support.run_unittest(FirstCase, debug)


if __name__ == "__main__":
    test_main()



class TestPath:
    _path_using_io = (
        'abspath', 'curdir', 'exists', 'expanduser', 'expandvars',
        'getatime', 'getctime', 'getmtime', 'getsize',
        'isfile', 'islink', 'ismount', 'realpath',
        'samefile', 'sameopenfile', 'samestat',
        'walk',

    )

    def __init__(self, os):
        for name in dir(os.path):
            if (not name.startswith('_') and
                    name not in self._path_using_io):
                setattr(self, name, getattr(os.path, name))


class TestIO:
    def __init__(self, mod):
        os = mod._root.os
        for name in mod.os_common:
            setattr(self, name, getattr(os, name))
        self.path = TestPath(os)
        self.files = {}
        self.tempno = 0

    def access(self, name, mode):
        if name in self.files:
            return True
        return False

    def listdir(self, name):
        li = []
        name = self.path.join(name, '')
        for k in self.files:
            if k.startswith(name):
                rest = k[len(name):]
                if rest:
                    li.append(rest)
        return li

    def mkdtemp(self):
        self.tempno += 1
        return '/tmp/xyz%d' % self.tempno

    def read_file(self, name):
        return self.files[name]

    def remove(self, name):
        try:
            del self.files[name]
        except KeyError:
            raise IOError('No such file: %r' % name)

    def rename(self, src, tgt):
        try:
            data = self.files[src]
        except KeyError:
            raise IOError('No such file: %r' % src)
        del self.files[src]
        self.files[tgt] = data

    def rmdir(self, name):
        pass

    def write_file(self, name, text):
        self.files[name] = text


class RealIO:
    def __init__(self, mod):
        os = mod._root.os
        for name in mod.os_common:
            setattr(self, name, getattr(os, name))
        self.path = os.path
        self.listdir = os.listdir
        self.makedirs = os.makedirs
        self.mkdtemp = mod._root.tempfile.mkdtemp
        self.rmdir = os.rmdir
        self.access = os.access
        self.chdir = os.chdir
        self.remove = os.remove
        self.rename = os.rename

    def read_file(self, name):
        with open(name) as f:
            return f.read()

    def write_file(self, name, data):
        with open(name, 'w') as f:
            f.write(data)


class _GLUECLAMP_:

    _setable_ = 'IO',

    os_common = ('R_OK', 'W_OK', 'X_OK')

    def _get_IO(self):
        return RealIO(self)

    def set_IO(self, IO):
        self.IO = IO

    def set_test_mode(self):
        self.set_IO(TestIO(self))



class FiniteAutomaton:
    def __init__(self, start_state, dump_state=''):
        self.start_state = start_state
        self.dump_state = dump_state
        self.table = {self.dump_state: {}}
        self.unresolved_composites = []
        self.resolved_composites = []
        self.composite_memo = {}
        self.is_updated = 1

    def __setitem__(self, xxx_todo_changeme, x):
        (state, symbol) = xxx_todo_changeme
        self.table.setdefault(state, {})[symbol] = x
        self.is_updated = 1

    def add_transition(self, state, symbol, *nexts):
        if len(nexts) == 1:
            c = nexts[0]
        else:
            for n in nexts:
                if n not in self.table:
                    self.table[n] = {}

            c = self.new_composite(nexts)
        self[state, symbol] = c
        if c not in self.table:
            self.table[c] = {}

    def get_row_items(self, state):
        try:
            x = list(self.table[state].items())
        except KeyError:
            return ()
        ris = []
        for k, v in x:
            if isinstance(v, CompositeState):
                for vi in v:
                    ris.append((k, vi))
            else:
                ris.append((k, v))
        return ris

    def make_deterministic(self):
        # Resolve all unresolved composite states so we become deterministic

        while self.unresolved_composites:
            composites = self.unresolved_composites
            self.unresolved_composites = []
            for c in composites:
                ds = {}
                l = []
                for a in c:
                    for k, v in self.get_row_items(a):
                        ds.setdefault(k, []).append(v)
                for k, v in list(ds.items()):
                    for a in c:
                        if k not in self.table[a]:
                            v.append(self.dump_state)
                            break
                for k, v in list(ds.items()):
                    self.add_transition(c, k, *v)

    def get_all_input_symbols(self):
        syms = {}
        for state, trans in list(self.table.items()):
            for k, v in trans:
                syms[k] = 1
        return syms

    def get_all_states(self):
        return self.table

    def get_all_final_states(self):
        return {}

    def get_composites(self):
        if self.is_updated:
            self.make_deterministic()
        return list(self.composite_memo.values())

    def get_transition_classes(self):
        # Get classes of states that have the same outgoing transitions
        tc = {}
        tck = {}
        for k, v in list(self.table.items()):
            trans = list(v.keys())
            trans.sort()
            trans = tuple(trans)
            ks = tc.get(trans)
            if ks is None:
                ks = []
                tc[trans] = ks
            ks.append(k)
            tck[k] = ks
        return tc, tck

    def get_minimized_dfa(self, finals):
        def markall(finals, tcv, table):
            def psrmark(l):
                for pq in l:
                    if pq in PS:
                        l1 = PS[pq]
                        del PS[pq]
                        if l1:
                            psrmark(l1)
            PS = {}
            for Q in tcv:
                for ip, p in enumerate(Q):
                    for q in Q[ip+1:]:
                        if (p in finals) == (q in finals):
                            PS[(p, q)] = ()

            for pq in list(PS.keys()):
                p, q = pq
                dp = table[p]
                dq = table[q]
                # Now we know they contain the same symbols
                dps = []
                for a in dp:
                    dpa = dp[a]
                    dqa = dq[a]
                    if dpa is not dqa:
                        dpadqa = (dpa, dqa)
                        if dpadqa in PS:
                            dps.append(dpadqa)
                        else:
                            dqadpa = (dqa, dpa)
                            if dqadpa in PS:
                                dps.append(dqadpa)
                            else:
                                l = PS[pq]
                                del PS[pq]
                                if l is not ():
                                    psrmark(l)
                                break
                else:
                    for dpadqa in dps:
                        l = PS[dpadqa]
                        if l is ():
                            l = [pq]
                            PS[dpadqa] = l
                        else:
                            l.append(pq)
            return PS

        def combine(QS, PQS):
            eqs = {}
            for Q in QS:
                for p in Q:
                    eqs[p] = [p]

            for p, q in PQS:
                #
                # Combine to equivalence classes
                #
                # Now we know that p, q are combinable
                #
                ep = eqs[p]
                eq = eqs[q]
                if eq is not ep:
                    if len(eq) > len(ep):
                        eq, ep = ep, eq
                    ep.extend(eq)
                    for q in eq:
                        eqs[q] = ep

            return eqs

        def finalize(eqs):
            csbyid = {}         # Composite state by identity of atomic state list
            csbyas = {}         # Composite state by atomic state

            i = 0
            for eq in list(eqs.values()):
                ideq = id(eq)
                if ideq not in csbyid:
                    c = 'MS%d' % i
                    i += 1
                    csbyid[ideq] = eq[0], c
                    for s in eq:
                        csbyas[s] = c

            fsa = self.__class__(csbyas[self.start_state])
            fsa.final_states = {}

            for cs0, cst in list(csbyid.values()):
                fsa.table[cst] = trans = {}
                for a, s in list(self.table[cs0].items()):
                    trans[a] = csbyas[s]
                if cs0 in finals:
                    fsa.final_states[cst] = 1

            return fsa

        if self.is_updated:
            self.make_deterministic()
            tctck = self._tctck = self.get_transition_classes()
            self.is_updated = 0
        else:
            tctck = self._tctck

        tc, tck = tctck
        tcv = list(tc.values())
        PS = markall(finals, tcv, self.table)
        eqs = combine(tcv, PS)
        fsa = finalize(eqs)
        return fsa

    def new_composite(self, args):
        cs = CompositeState(list(dict([(arg, 1) for arg in args]).keys()))
        if len(cs) == 1:
            return args[0]
        try:
            return self.composite_memo[cs]
        except KeyError:
            self.composite_memo[cs] = cs
            self.unresolved_composites.append(cs)
            return cs

    def pp(self):
        ks = list(self.table.keys())
        ks.sort()
        num = dict([(s, i) for i, s in enumerate(ks)])
        for s in ks:
            k = self.table[s]
            print('%d: %s' % (num[s], s))
            cs = list(k.keys())
            cs.sort()
            for c in cs:
                v = k[c]
                print('   %r  -> #%d: %s' % (c, num[v], v))


class CompositeState(tuple):
    pass



from guppy.heapy.test import support


class FirstCase(support.TestCase):
    def setUp(self):
        support.TestCase.setUp(self)

    def test_1(self):
        asrt = self.assertTrue
        a = []
        b = []
        c = self.iso(a, b)
        asrt(len(c.nodes) == 2)
        asrt(a in c)
        asrt(b in c)
        asrt([] not in c)
        asrt(c not in c)

        d = self.idset(c.nodes)

        asrt(c.nodes == d.nodes)

        asrt(c == d)

    def test_2(self):
        # Test standard set operations
        H = self.idset
        e1 = []
        e2 = {}
        e3 = ()
        la = [], [e1], [e1, e2], [e1, e2, e3], [e2], [e2, e3], [e3]
        self.guppy.sets.test.test_set_operations(
            [H(x) for x in la], [H(x) for x in la], [H(x) for x in la])

    def test_3(self):
        # Test out-reaching

        iso = self.iso

        a = []
        b = [a]
        c = [b]
        self.View.root = c

        x = iso(b)

        self.assertTrue(x.referrers == iso(c))
        self.aseq(x.referents,  iso(a))
        self.aseq(x.referents.referrers,  x)
        self.aseq(x.dominos, iso(a, b))


def test_main(debug=False):
    support.run_unittest(FirstCase, debug)


if __name__ == "__main__":
    test_main()



import functools

from guppy.etc.Descriptor import property_nondata


class R_NORELATION:
    code = -1
    r = None

    def stra(self, a, safe=True):
        return '%s.??' % a


class R_IDENTITY:
    code = 0

    def stra(self, a, safe=True):
        return a


class R_ATTRIBUTE:
    code = 1
    strpat = '%s.%s'


class R_INDEXVAL:
    code = 2

    def stra(self, a, safe=True):
        if safe:
            return '%s[%s]' % (a, self.saferepr(self.r))
        else:
            return '%s[%r]' % (a, self.r)


class R_INDEXKEY:
    code = 3
    strpat = '%s.keys()[%r]'


class R_INTERATTR:
    code = 4
    strpat = '%s->%s'


class R_HASATTR:
    code = 5
    strpat = '%s.__dict__.keys()[%r]'


class R_LOCAL_VAR:
    code = 6
    strpat = '%s.f_locals[%r]'


class R_CELL:
    code = 7
    strpat = '%s.f_locals [%r]'


class R_STACK:
    code = 8
    strpat = '%s->f_valuestack[%d]'


class R_INSET:
    code = 9
    strpat = 'list(%s)[%d]'


class R_RELSRC:
    code = 10

    def stra(self, a, safe=True):
        return self.r % (a,)


class R_LIMIT:
    code = 11


@functools.total_ordering
class RelationBase(object):
    __slots__ = 'r', 'isinverted'

    def __init__(self, r, isinverted=0):
        self.r = r
        self.isinverted = isinverted

    def __lt__(self, other):
        if isinstance(other, RelationBase):
            if self.code != other.code:
                return self.code < other.code
            return self.r < other.r
        else:
            return id(type(self)) < id(type(other))

    def __eq__(self, other):
        if isinstance(other, RelationBase):
            if self.code != other.code:
                return False
            return self.r == other.r
        else:
            return False

    def __str__(self):
        return self.stra('%s')

    def inverted(self):
        return self.__class__(self.r, not self.isinverted)

    def stra(self, a, safe=True):
        return self.strpat % (a, self.r)


class MultiRelation(RelationBase):
    def __init__(self, rels):
        self.rels = rels

    def stra(self, a, safe=True):
        return '<'+','.join([x.stra(a, safe=safe) for x in self.rels])+'>'


@functools.total_ordering
class Path:
    def __init__(self, mod, path, index, srcname):
        self.mod = mod
        self.path = path[1:]
        self.index = index
        self.src = path[1]
        self.tgt = path[-1]
        self.strprefix = '%s'
        if srcname == '_str_of_src_':
            srcname = self.src.brief
        if callable(srcname):
            srcname = srcname(self)
        self.srcname = srcname

    def __lt__(self, other):
        return str(self) < str(other)

    def __eq__(self, other):
        return str(self) == str(other)

    def __len__(self):
        return int((len(self.path) - 1) / 2)

    def stra(self, safe=True):
        if self.path:
            s = self.strprefix
            for i in range(1, len(self.path), 2):
                r = self.path[i]
                s = r.stra(s, safe=safe)
        else:
            s = '<Empty Path>'
        return s

    def __str__(self):
        return self.stra()

    def __repr__(self):
        return repr(str(self))

    def _get_line_iter(self):
        yield '%2d: %s' % (self.index, str(self) % self.srcname)

    def types(self):
        return [type(x) for x in self.path]


class PathsIter:
    def __init__(self, paths, start=None, stop=None):
        self.paths = paths
        self.mod = paths.mod
        self.stop = stop
        self.reset(start)

    def __iter__(self):
        return self

    def reset(self, idx=None):
        if idx is None:
            idx = 0
        if idx != 0:  # Optimization: don't calculate numpaths in common case.
            ln = self.paths.numpaths
            if idx < 0:
                idx = ln + idx
            if not (0 <= idx < ln):
                self.isatend = 1
                return
        Src = self.paths.Src
        sr = [('%s', src.by(Src.er)) for src in Src.byid.parts]
        srs = []
        idxs = []
        np = 0
        while sr:
            if idx == 0:
                i, (rel, src) = 0, sr[0]
            else:
                for i, (rel, src) in enumerate(sr):
                    npnext = np + self.paths.numpaths_from(src)
                    if idx < npnext:
                        break
                    np = npnext
                else:
                    assert 0
            idxs.append(i)
            srs.append(sr)
            sr = self.mod.sortedrels(self.paths.IG, src)
        self.pos = idx
        self.idxs = idxs
        self.srs = srs
        self.isatend = not idxs

    def __next__(self):
        paths = self.paths
        if (self.isatend or
                self.stop is not None and self.pos >= self.stop):
            raise StopIteration
        path = []
        for row, col in enumerate(self.idxs):
            sr = self.srs[row]
            if sr is None:
                sr = self.mod.sortedrels(paths.IG, path[-1])
                self.srs[row] = sr
            rel, dst = sr[col]
            path.append(rel)
            path.append(dst)
        rp = self.mod.Path(paths, path, self.pos, paths.srcname)
        self.pos += 1
        while row >= 0:
            self.idxs[row] += 1
            if self.idxs[row] < len(self.srs[row]):
                break
            if row > 0:
                self.srs[row] = None
            self.idxs[row] = 0
            row -= 1
        else:
            self.isatend = 1
            self.pos = 0
        return rp


class ShortestPaths:
    def __init__(self, sg, Dst):
        self.sg = sg
        self.Dst = Dst
        self.mod = mod = sg.mod
        self._hiding_tag_ = mod._hiding_tag_
        self.srcname = sg.srcname
        self.top = self

        self.IG = IG = mod.nodegraph()
        Edges = []
        Y = Dst.nodes
        while Y:
            R = sg.G.domain_restricted(Y)
            R.invert()
            IG.update(R)
            Edges.append(R)
            Y = R.get_domain()
        if Edges:
            Edges.pop()
            Edges.reverse()
            self.Src = mod.idset(Edges[0].get_domain())
        else:
            self.Src = mod.iso()

        self.edges = tuple(Edges)
        sets = []
        for i, e in enumerate(Edges):
            if i == 0:
                sets.append(mod.idset(e.get_domain()))
            sets.append(mod.idset(e.get_range()))
        self.sets = tuple(sets)

        mod.OutputHandling.setup_printing(self)

        self.maxpaths = 10

    def __getitem__(self, idx):
        try:
            return next(self.iter(start=idx))
        except StopIteration:
            raise IndexError

    def __iter__(self):
        return self.iter()

    def iter(self, start=0, stop=None):
        return PathsIter(self, start, stop)

    def aslist(self):
        return list(self)

    def copy_but_avoid_edges_at_levels(self, *args):
        avoid = self.edges_at(*args).updated(self.sg.AvoidEdges)
        assert avoid._hiding_tag_ is self.mod._hiding_tag_
        return self.mod.shpaths(self.Dst, self.Src, avoid_edges=avoid)
        # return self.mod.shpaths(self.dst, self.src, avoid_edges=avoid)

    avoided = copy_but_avoid_edges_at_levels

    # The builtin __len__ doesn't always work due to builtin Python restriction to int result:
    # so we don't provide it at all to avoid unsuspected errors sometimes.
    # Use .numpaths attribute instead.
    #   def __len__(self):
    #      return self.numpaths

    def depth(self):
        pass

    def edges_at(self, *args):
        E = self.mod.nodegraph()
        for col in args:
            E.update(self.edges[col])
        assert E._hiding_tag_ == self.mod._parent.View._hiding_tag_
        return E

    def numpaths_from(self, Src):
        try:
            NP = self.NP
        except AttributeError:
            NP = self.mod.nodegraph(is_mapping=True)
            NP.add_edges_n1(self.IG.get_domain(), None)
            for dst in self.Dst.nodes:
                NP.add_edge(dst, 1)
            self.NP = NP
        numedges = self.mod.hv.numedges
        IG = self.IG

        def np(y):
            n = NP[y]
            if n is None:
                n = 0
                for z in IG[y]:
                    sn = NP[z]
                    if sn is None:
                        sn = np(z)
                    n += sn * numedges(y, z)
                NP[y] = n
            return n
        num = 0
        for src in Src.nodes:
            num += np(src)
        return num

    def _get_numpaths(self):
        num = self.numpaths_from(self.Src)
        self.numpaths = num
        return num

    numpaths = property_nondata(fget=_get_numpaths)

    @property
    def maxpaths(self):
        return self.printer.max_more_lines

    @maxpaths.setter
    def maxpaths(self, value):
        self.printer.max_more_lines = value

    def _oh_get_num_lines(self):
        return self.numpaths

    def _oh_get_line_iter(self):
        for el in self:
            yield from el._get_line_iter()

    def _oh_get_more_msg(self, start_lineno, end_lineno):
        nummore = self.numpaths-(end_lineno+1)
        return '<... %d more paths ...>' % nummore

    def _oh_get_empty_msg(self):
        if self.numpaths:
            return '<No more paths>'
        return None


class ShortestGraph:
    def __init__(self, mod, G, DstSets, Src, AvoidEdges,
                 srcname=None, dstname=None):
        self.mod = mod
        self.G = G
        self.Src = Src
        self.DstSets = DstSets
        self.AvoidEdges = AvoidEdges
        if srcname is None:
            if Src.count == 1:
                srcname = mod.srcname_1
            else:
                srcname = mod.srcname_n
        self.srcname = srcname
        if dstname is None:
            dstname = mod.dstname
        self.dstname = dstname

    def __getitem__(self, idx):
        return self.mod.ShortestPaths(self, self.DstSets[idx])

    def __len__(self):
        return len(self.DstSets)

    def __repr__(self):
        lst = []
        for i, p in enumerate(self):
            lst.append('--- %s[%d] ---' % (self.dstname, i))
            lst.append(str(p))

        return '\n'.join(lst)


class _GLUECLAMP_:
    _preload_ = ('_hiding_tag_',)
    _chgable_ = ('output', 'srcname_1', 'srcname_n')

    srcname_1 = 'Src'
    srcname_n = '_str_of_src_'
    dstname = 'Dst'

    _imports_ = (
        '_parent.ImpSet:mutnodeset',
        '_parent:OutputHandling',
        '_parent.Use:idset',
        '_parent.Use:iso',
        '_parent.Use:Nothing',
        '_parent.Use:reprefix',
        '_parent.UniSet:idset_adapt',
        '_parent.View:hv',
        '_parent.View:nodegraph',
        '_parent:View',  # NOT View.root, since it may change
    )

    def _get_rel_table(self):
        table = {}
        for name in dir(self._module):
            if name.startswith('R_'):
                c = getattr(self, name)

                class r(c, self.RelationBase):
                    repr = self.saferepr
                    saferepr = self.saferepr
                r.__qualname__ = r.__name__ = 'Based_'+name
                table[c.code] = r
        return table

    def _get__hiding_tag_(self): return self._parent.View._hiding_tag_
    def _get_identity(self): return self.rel_table[R_IDENTITY.code]('')
    def _get_norelation(self): return self.rel_table[R_NORELATION.code]('')
    def _get_saferepr(self): return self._root.reprlib.repr
    def _get_shpathstep(self): return self.hv.shpathstep

    def sortedrels(self, IG, Src):
        t = []
        iso = self.iso
        for src in Src.nodes:
            for dst in IG[src]:
                Dst = iso(dst)
                for rel in self.relations(src, dst):
                    t.append((rel, Dst))
        t.sort(key=lambda x: x[0])
        return t

    def prunedinverted(self, G, Y):
        IG = self.nodegraph()
        while Y:
            R = G.domain_restricted(Y)
            R.invert()
            IG.update(R)
            Y = R.get_domain()
        return IG

    def relation(self, src, dst):
        tab = self.relations(src, dst)
        if len(tab) > 1:
            r = MultiRelation(tab)
        elif not tab:
            r = self.norelation
        else:
            r = tab[0]
        return r

    def relations(self, src, dst):
        tab = []
        if src is dst:
            tab.append(self.identity)
        rawrel = self.hv.relate(src, dst)
        for i, rs in enumerate(rawrel):
            for r in rs:
                tab.append(self.rel_table[i](r))
        if not tab:
            tab = [self.norelation]
        return tab

    def shpaths(self, dst, src=None, avoid_nodes=None, avoid_edges=()):
        return self.shpgraph([dst], src, avoid_nodes, avoid_edges)[0]

    def shpgraph(self, DstSets, src=None, avoid_nodes=None, avoid_edges=(),
                 srcname=None, dstname=None):
        if src is None:
            Src = self.iso(self.View.root)
            if srcname is None and self.View.root is self.View.heapyc.RootState:
                srcname = '%sRoot' % self.reprefix
        else:
            Src = self.idset_adapt(src)
        if avoid_nodes is None:
            AvoidNodes = self.Nothing
        else:
            AvoidNodes = self.idset_adapt(avoid_nodes)
        AvoidEdges = self.nodegraph(avoid_edges)
        G, DstSets = self.shpgraph_algorithm(
            DstSets, Src, AvoidNodes, AvoidEdges)

        return self.ShortestGraph(self, G, DstSets, Src, AvoidEdges,
                                  srcname, dstname)

    def shpgraph_algorithm(self, DstSets, Src, AvoidNodes, AvoidEdges):
        U = (Src - AvoidNodes).nodes
        S = self.mutnodeset(AvoidNodes.nodes)
        G = self.nodegraph()
        unseen = list(enumerate(DstSets))
        DstSets = [self.Nothing]*len(DstSets)
        while U and unseen:
            S |= U
            U = self.shpathstep(G, U, S, AvoidEdges)
            unseen_ = []
            for i, D in unseen:
                D_ = D & U
                if D_:
                    DstSets[i] = D_
                else:
                    unseen_.append((i, D))
            unseen = unseen_
        return G, [self.idset_adapt(D) for D in DstSets]


class _Specification_:
    class GlueTypeExpr:
        exec("""\
shpgraph        <in>    callable
""".replace('<in>', ' = lambda IN : '))



import re
import sys
import types


class GlueError(Exception):
    pass


class RecursionError(GlueError):
    pass


class NoSuchAttributeError(GlueError):
    pass


def ispackage(m):
    """ Determine if a module is a package - that means, sub-modules can be imported
    Currently uses that it has a file name that matches '.*__init__.py[co]?$'
    xxx is this portable/future-safe?

    """
    try:
        name = m.__file__
    except AttributeError:
        return 0
    return re.match('.*__init__.py[co]?$', name)


def dotname(first, last):
    if first and last:
        return '%s.%s' % (first, last)
    else:
        return first + last


class Interface(object):
    def __init__(self, share, owner, name):
        self.__dict__['_share'] = share
        self.__dict__['_owner'] = owner
        self.__dict__['_name'] = name
        for name in share.preload:
            getattr(self, name)

    def _import(self, *names):
        return ','.join(names) + '=' + ','.join(['self._root.%s' % name for name in names])

    def __getattr__(self, name):
        return self._share.getattr(self, name)

    def __setattr__(self, name, value):
        return self._share.setattr(self, name, value)


class Owner:
    def __init__(self, name):
        self.name = name
        self.galog = {}
        self.salog = {}
        self.inters = {}

    def log_getattr(self, cache, name):
        name = dotname(cache['_name'], name)
        self.galog[name] = 1

    def log_setattr(self, name):
        self.salog[name] = 1

    def makeInterface(self, cache, share, name):
        name = dotname(cache['_name'], name)
        if share not in self.inters:
            Clamp = share.Clamp
            if Clamp is not None and issubclass(Clamp, Interface):
                NewInterface = Clamp
            else:
                NewInterface = Interface
            self.inters[share] = NewInterface(share, self, name)
        return self.inters[share]

    def pp(self, out=None, short=0):
        if out is None:
            out = sys.stdout
        if not short:
            print('Attributes used by %s:' % self.name, file=out)
            print(self.name[:self.name.rindex('.')+1], file=out)
        complete = []
        for a in self.galog:
            for b in self.galog:
                if a != b and b.startswith(a):
                    break
            else:
                complete.append(a)
        complete.sort()
        for a in complete:
            print('    ', a)


class Share:
    has_getattr_logging_enabled = False
    Clamp = None

    def __init__(self, module, parent, name, Clamp):
        if parent is None:
            parent = self
            root = self
        else:
            root = parent.data['_root']
        self.module = module
        self.parent = parent
        self.name = name
        if Clamp is not None:
            self.Clamp = Clamp

        self.setable = getattr(Clamp, '_setable_', ())
        if not isinstance(self.setable, tuple):
            raise TypeError(self.message(
                'the _setable_ attribute must be a tuple'))

        self.chgable = getattr(Clamp, '_chgable_', ())
        if not isinstance(self.chgable, tuple):
            raise TypeError(self.message(
                'the _chgable_ attribute must be a tuple'))

        if Clamp is not None:
            for attr in Clamp.__dict__:
                if attr.startswith('_set_'):
                    attr = attr[len('_set_'):]
                    if attr not in self.setable and attr not in self.chgable:
                        raise TypeError(self.message(
                            '%s must be in either _setable_ or _chgable_ '
                            'for _set_%s to work' % (attr, attr)))

        imports = getattr(Clamp, '_imports_', ())
        if not isinstance(imports, tuple):
            raise TypeError(self.message(
                'the _imports_ attribute must be a tuple'))
        self.importedfrom = {}
        pres = {}
        parent_inter = root_inter = None
        for fi in imports:
            presuf = fi.split(':')
            if len(presuf) != 2:
                if len(presuf) > 2:
                    s = "Too many"
                else:
                    s = "No"
                raise SyntaxError("%s ':' in import directive %r." % (s, fi))
            pre, suf = presuf
            pre = pre.strip()
            prepa = pres.get(pre)
            if prepa is None:
                hdta = pre.split('.', 1)
                hd = hdta[0]
                if hd == '_parent':
                    if parent_inter is None:
                        parent_inter = Owner('').makeInterface(
                            {'_name': ''}, parent, '')
                    hdo = parent_inter
                elif hd == '_root':
                    if root_inter is None:
                        root_inter = Owner('').makeInterface(
                            {'_name': ''}, root, '')
                    hdo = root_inter
                else:
                    raise SyntaxError(
                        "Module must begin with _parent or _root")
                if len(hdta) == 2:
                    prepa = [hdo, hdta[1], None]
                else:
                    prepa = [hdo, '', hdo]
                pres[pre] = prepa

            sufs = suf.split(',')
            for su in sufs:
                su = su.strip()
                im = getattr(Clamp, '_get_%s' % su, None)
                if im is not None:
                    raise ValueError(
                        'Cant have both name (=%r) in boht importfrom  and _get' % su)

                self.importedfrom[su] = prepa

        self.nowrap = getattr(Clamp, '_nowrap_', ())
        if not isinstance(self.nowrap, tuple):
            raise TypeError(self.message(
                'the _nowrap_ attribute must be a tuple'))
        wrapattr = getattr(Clamp, '_wrapattr_', None)
        if not isinstance(wrapattr, types.FunctionType) and wrapattr is not None:
            raise TypeError(self.message(
                'the _wrapattr_ attribute must be a method'))
        self.wrapattr = wrapattr
        self.wrapping = 0
        self.data = {}
        self.owners = {}
        self.ispackage = module is None or ispackage(module)
        self.data['_parent'] = parent
        self.data['_root'] = root
        self.data['_module'] = module
        self.recursion = 0

        preload = getattr(Clamp, '_preload_', ())
        if preload:
            self.preload = ()
            inter = Owner('').makeInterface({'_name': ''}, self, '')
            for name in preload:
                getattr(inter, name)
        for name in preload:
            assert name in self.data
        self.preload = preload

    def message(self, msg):
        return '%s: in %r: %s' % (self.__class__, self.name, msg)

    def getattr(self, inter, name):
        owner = inter._owner
        cache = inter.__dict__
        d = self.getattr2(inter, cache, owner, name)
        if name not in self.chgable:
            cache[name] = d
        return d

    def getattr2(self, inter, cache, owner, name):
        if self.has_getattr_logging_enabled:
            owner.log_getattr(cache, name)

        if name not in self.data:
            try:
                self.recursion += 1
                try:
                    if self.ispackage:
                        try:
                            x = self.getattr3(inter, name)
                        except NoSuchAttributeError:
                            x = self.getattr_package(inter, name)
                    else:
                        x = self.getattr3(inter, name)
                except NoSuchAttributeError:
                    if name == '__repr__':
                        return lambda: str(inter)
                    elif name == '__str__':
                        return lambda: '<%s interface at %s>' % (inter._name,
                                                                 hex(id(self)))
                    else:
                        x = self.getattr_module(inter, name)
                wrapattr = self.wrapattr
                if wrapattr is not None and name not in self.nowrap:
                    assert not self.wrapping
                    try:
                        self.wrapping = 1
                        x = wrapattr(inter, x, name)
                    finally:
                        self.wrapping = 0
                self.data[name] = x
            finally:
                self.recursion -= 1

        x = self.data[name]
        if isinstance(x, Share):
            x = owner.makeInterface(cache, x, name)
        return x

    def getattr_module(self, inter, name):
        try:
            return getattr(self.module, name)
        except AttributeError:
            raise AttributeError('Module %r or its _GLUECLAMP_ has no attribute %r' % (
                self.module.__name__, name))

    def getattr_package(self, inter, name):
        try:
            x = __import__(self.makeName(name))
        except ImportError as exc:
            if (exc.args[0] != 'No module named \'%s\'' % self.makeName(name) or
                    (not self.module and name not in ('__repr__', '__str__'))):
                raise
            x = self.getattr3(inter, name)
            # raise AttributeError, name
        else:
            if self.module:
                x = self.getattr_module(inter, name)
        if isinstance(x, types.ModuleType):
            x = self.makeModule(x, name)
        return x

    def getattr3(self, inter, name):
        if self.recursion >= 10:
            raise RecursionError(name)
        Clamp = self.Clamp
        if Clamp is None:
            raise NoSuchAttributeError(name)

        try:
            x = getattr(Clamp, name)
        except AttributeError:
            pass
        else:
            if isinstance(x, types.FunctionType):
                x = types.MethodType(x, inter)
            return x

        try:
            im = getattr(Clamp, '_get_%s' % name)
        except AttributeError:
            pass
        else:
            owner = self.makeOwner(name)
            inter = Interface(self, owner, '')
            f = types.MethodType(im, inter)
            x = f()
            if isinstance(x, Interface):
                x = x.__dict__['_share']
            return x

        if name in self.importedfrom:
            prepa = self.importedfrom[name]
            hdo, ta, pa = prepa
            if pa is None:
                pa = hdo
                tas = ta.split('.')
                for at in tas:
                    pa = getattr(pa, at)
                prepa[2] = pa
            x = getattr(pa, name)
            return x

        gp = getattr(Clamp, '_GLUEPATH_', None)
        if gp is None:
            raise NoSuchAttributeError(name)
        if hasattr(gp, 'split'):
            gp = gp.split(',')
        for a in gp:
            a = a.strip()
            bs = a.split('.')
            ii = inter
            for b in bs:
                b = b.strip()
                ii = getattr(ii, b)
            try:
                x = getattr(ii, name)
            except AttributeError:
                continue
            else:
                return x
        raise NoSuchAttributeError(name)

    def makeModule(self, module, name):
        Clamp = getattr(module, '_GLUECLAMP_', None)
        return Share(module, self, module.__name__, Clamp)

    def makeName(self, name):
        if self.name:
            name = '%s.%s' % (self.name, name)
        return name

    def makeOwner(self, name):
        owner_name = self.makeName(name)
        owner = Owner(owner_name)
        self.owners[owner_name] = owner
        return owner

    def pp(self, out=sys.stdout):
        if not self.owners:
            return
        print('Dependencies found for %s' % self.name, file=out)
        print('-----------------------'+'-'*len(self.name), file=out)
        keys = list(self.owners.keys())
        keys.sort()
        for key in keys:
            lastname = key[key.rindex('.')+1:]
            print(lastname, file=out)
            self.owners[key].pp(short=1)
        print(file=out)

    def rpp(self, out=sys.stdout):
        self.pp(out)
        for k, d in list(self.data.items()):
            if k not in ('_root', '_parent'):
                if isinstance(d, Share):
                    d.rpp()

    def setattr(self, inter, name, value):
        Clamp = self.Clamp
        if Clamp is None:
            raise ValueError(
                'Can not change attribute %r because no _GLUECLAMP_ defined.' % name)

        setable = self.setable
        chgable = self.chgable
        if (name not in setable and name not in chgable and
                (not (name in self.data and self.data[name] is value))):
            raise ValueError("""Can not change attribute %r,
because it is not in _setable_ or _chgable_.""" % name)
        if name in self.data and self.data[name] is not value and name not in chgable:
            raise ValueError("""Can not change attribute %r,
because it is already set and not in _chgable_.""" % name)

        im = getattr(Clamp, '_set_%s' % name, None)
        if im is not None:
            im(inter, value)

        self.data[name] = value
        if name not in chgable:  # This is a pain, I suppose. Should we track interfaces?
            inter.__dict__[name] = value


class Test:
    def __init__(self, root):
        self.root = root


class _GLUECLAMP_:
    pass


def Root():
    """\
Create a new guppy Root object.

All functionality in the system may be accessed from this object.
Modules are imported on demand when accessed. Other objects may be
created or imported on demand using Guppy Glue+ directives.
"""
    share = Share(None, None, '', None)
    r = Owner('').makeInterface({'_name': ''}, share, '')
    share.root_interface = r
    return r



class _GLUECLAMP_:
    _imports_ = (
        '_root:os',
        '_root:webbrowser',
    )

    default_doc_file = 'guppy.html'

    def doc(self, subject=None, *args, **kwds):
        """\
        This doesnt work well or at all
        There are painful were where-to-find the files issues
        for the distributed and installed package.
        for I couldn't have the data among the modules themselves.

        Get documentation about a subject or generally about the Guppy system.
        It will show the documentation in the system web browser.
        If the subject argument is an object that the documentation system
        recognizes, it will bring up the documentation for that kind of object.
        Otherwise it will bring up a general documentation page.
        """

        self.doc_default()

    def doc_default(self):
        self.open_local_filename(self.default_doc_file)

    def open_local_filename(self, filename):
        self.webbrowser.open(self.os.path.join(self.doc_dir, filename))



from tkinter import *
import sys
import gc


class FixedMenu(Menu):
    # A fix for the .delete() method in Menu.
    # To delete commands defined in the menu items deleted.
    # Also changed the comment: INDEX2 is actually INCLUDED.
    def delete(self, index1, index2=None):
        """Delete menu items between INDEX1 and INDEX2 (included)."""
        print(self._tclCommands)
        if index2 is None:
            index2 = index1
        # First find out what entries have defined commands.
        cmds = []
        for i in range(self.index(index1), self.index(index2)+1):
            c = str(self.entrycget(i, 'command'))
            if c in self._tclCommands:
                # I don't want to delete the command already, since it
                # seems mystical to do that while the entry is not yet deleted.
                cmds.append(c)
        # Delete the menu entries.
        self.tk.call(self._w, 'delete', index1, index2)
        # Now that the menu entries have been deleted, we can delete their commands.
        for c in cmds:
            self.deletecommand(c)


def test1(M):
    # Test with a single command
    gc.collect()
    root = Tk()
    button = Menubutton(root, text='Window')
    menu = M(button)
    button['menu'] = menu

    def command():
        print('command button pressed')
    rc = sys.getrefcount(command)
    menu.add_command(command=command)  # or add_radiobutton etc
    idx = menu.index(END)
    menu.delete(idx)
    gc.collect()
    rc1 = sys.getrefcount(command)
    print('leak test with class', M, end=' ')
    if rc1 != rc:
        print('failed: command is now hold by', rc1, 'references')
    else:
        print('succeeded: command is now hold by', rc1, 'references')

    root.destroy()


def test2(M):
    # Test with 3 commands, especially to see that deleting a range works.

    gc.collect()
    root = Tk()
    button = Menubutton(root, text='Window')
    menu = M(button)
    button['menu'] = menu

    def command0():
        print('command 0 button pressed')
        'deleting 0 and 1'
        menu.delete(idx0, idx1)

    def command1():
        print('command 1 button pressed')

    def command2():
        print('command 2 button pressed')
        print('deleting at END')
        menu.delete(END)
        root.quit()
    rc = [sys.getrefcount(x) for x in (command0, command1, command0)]
    button.pack()
    # or add_radiobutton etc
    menu.add_command(command=command0, label='press first')
    idx0 = menu.index(END)
    menu.add_radiobutton(command=command1, label='command1')
    # to see that delete works even when no command supplied
    menu.add_command(label='no Command')
    idx1 = menu.index(END)
    menu.add_command(command=command2, label='press last')
    idx2 = menu.index(END)
    root.mainloop()

    gc.collect()
    rc1 = [sys.getrefcount(x) for x in (command0, command1, command0)]
    print('leak test with class', M, end=' ')
    if rc1 != rc:
        print('failed: command is now hold by',
              rc1, 'references, should be', rc)
    else:
        print('succeeded: command is now hold by', rc1, 'references')

    root.destroy()


for M in (Menu, FixedMenu,):
    test1(M)
    test2(M)



# A Tk window that shows what cursor shapes are available.
# Moving the mouse over the cursor name shows the cursor in that shape.

from tkinter import *


def tkcursors(master=None):
    if master is None:
        root = Tk()
    else:
        root = master
    for i, cursor in enumerate((
        'X_cursor',
        'arrow',
        'based_arrow_down',
        'based_arrow_up',
        'boat',
        'bogosity',
        'bottom_left_corner',
        'bottom_right_corner',
        'bottom_side',
        'bottom_tee',
        'box_spiral',
        'center_ptr',
        'circle',
        'clock',
        'coffee_mug',
        'cross',
        'cross_reverse',
        'crosshair',
        'diamond_cross',
        'dot',
        'dotbox',
        'double_arrow',
        'draft_large',
        'draft_small',
        'draped_box',
        'exchange',
        'fleur',

        'gobbler',
        'gumby',
        'hand1',
        'hand2',
        'heart',
        'icon',
        'iron_cross',
        'left_ptr',
        'left_side',
        'left_tee',
        'leftbutton',
        'll_angle',
        'lr_angle',
        'man',
        'middlebutton',
        'mouse',
        'pencil',
        'pirate',
        'plus',
        'question_arrow',
        'right_ptr',
        'right_side',
        'right_tee',
        'rightbutton',
        'rtl_logo',
        'sailboat',
        'sb_down_arrow',

        'sb_h_double_arrow',
        'sb_left_arrow',
        'sb_right_arrow',
        'sb_up_arrow',
        'sb_v_double_arrow',
        'shuttle',
        'sizing',
        'spider',
        'spraycan',
        'star',
        'target',
        'tcross',
        'top_left_arrow',
        'top_left_corner',
        'top_right_corner',
        'top_side',
        'top_tee',
        'trek',
        'ul_angle',
        'umbrella',
        'ur_angle',
        'watch',
        'xterm'


    )):

        w = Label(root, text=cursor, cursor=cursor,
                  width=20, anchor=W, relief=SUNKEN)
        col, row = divmod(i, 27)
        w.grid(row=row, column=col)
    if master is None:
        root.mainloop()


if __name__ == '__main__':
    tkcursors()



import functools
import inspect


class property_nondata:
    '''@property, but using non-data descriptor protocol'''
    def __init__(self, fget):
        self.fget = fget
        functools.update_wrapper(self, fget)

    def __get__(self, instance, owner=None):
        return self.fget(instance)


class property_exp(property):
    '''@property, but blacklist tab completers like rlcompleter from getattr'''
    def __init__(self, fget, *, doc=None):
        super().__init__(fget)
        self.__doc__ = doc

    def __get__(self, instance, owner=None):
        try:
            frame = inspect.currentframe()
            try:
                frame = frame.f_back
                if frame.f_globals['__name__'] == 'rlcompleter':
                    return None
            finally:
                del frame
        except Exception:
            pass
        return super().__get__(instance, owner)



class Node2XHTML:
    def __init__(self, mod, node=None, error_report=None, encode_name=None
                 ):
        self.mod = mod
        self.valid_html40 = False

        self.encode = self.mod.encode
        if encode_name is None:
            encode_name = self.mod.encode_name
        self.encode_name = encode_name
        if error_report is not None:
            self.error_report = error_report

        self.document_lang = None

        self.header_nodes = []

        self.indent = 0
        self.indentstep = 1

        self.set_out([])

        # xxx where do this?
        charset = 'utf-8'
        self.header_nodes.append(self.mod.node_of_taci(
            'meta', '', (
                self.mod.node_of_taci('http-equiv=', 'Content-Type'),
                self.mod.node_of_taci('content=',
                                      'text/html; charset=%s' % charset))))

        if node is not None:
            node.accept(self)

    def _visit_children(self, node):
        node, attrs = node.split_attrs()
        # xxx handle attrs?
        E = self.mod.ReportedError
        for ch in node.children:
            try:
                ch.accept(self)
            except E:
                pass

    def begin(self, tag, arg=''):
        t = '<'+tag
        if arg:
            t = t + ' ' + arg
        if tag in self.mod._no_end_tag_elements:
            # Quote from: http://gutenberg.hwg.org/markupXHTML.html
            #     N.B. The penultimate closing slash on empty elements such as the <img/>
            #     element can cause a problem in older browsers. For this reason it is
            #     recommended that you leave a space before the slash, namely <img />
            t += ' /'
        t += '>'
        if tag in self.mod.line_break_allowed:
            t = '\n'+self.indent * ' ' + t
        self.append(t)
        self.indent += self.indentstep

    def chg_out(self, out):
        oo = self.out
        self.set_out(out)
        return oo

    def encode_link_name(self, name):
        # 1. Make the name better looking for a html user's perspective
        # 2. Encode it by HTML rules

        if name.startswith(self.mod.tgt_prefix):
            name = name[len(self.mod.tgt_prefix):]
        else:
            # Should not happen often or at all
            assert 0
        name = self.encode_name(name)
        return name

    def end(self, tag):
        self.indent -= self.indentstep
        self.append('</%s>' % tag)

    def error(self, msg, *args, **kwds):
        msg = 'Doc2XHTML: ' + msg
        self.error_report(msg, *args, **kwds)

    def error_report(self, msg, *args, **kwds):
        print('HTML ENCODING ERROR: ', msg, 'args=', args, 'kwds=', kwds)
        raise ValueError

    def gen_document_header(self, lang, header_nodes):
        # lang & title are nodes with text or char directives, to be encoded.
        # metas is a list of nodes, with data to be encoded

        strict = 1  # we have alternatives, I just havent yet decided how or if to let the user choose

        if strict:
            self.append("""\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
""")
        else:
            self.append("""\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
""")

        self.begin('html',
                   'lang=%r xmlns="http://www.w3.org/1999/xhtml"' % self.get_encoded_text(
                       lang),
                   )
        self.begin('head')
        for node in header_nodes:
            self.gen_stdhtml(node)

        self.end('head')
        self.begin('body')

        # Get around w3c restriction that character data are not allowed
        # directly in body, makes it easier to write compliant code
        # Arguably the restriction is there for a reason, but I dont know...
        self.begin('div')

    def gen_document_trailer(self):
        self.end('div')
        self.end('body')
        self.end('html')

    def gen_empty_elmt(self, tag, arg=''):
        self.begin(tag, arg)
        self.indent -= self.indentstep

    def gen_generated_from_gsl(self):
        self.gen_empty_elmt('hr')

        self.append('Generated by ')
        self.begin('a', 'href="https://zhuyifei1999.github.io/guppy3/gsl.html"')
        #self.begin('a', 'href="gsl.html"')
        self.append('GSL-XHTML 0.1.7')
        self.end('a')
        self.append(' on '+self.mod.time.asctime(self.mod.time.localtime()))

    def gen_meta(self, node, tag=None):
        mknode = self.mod.node_of_taci
        if tag is None:
            tag = node.tag
        self.header_nodes.append(
            mknode('meta', '',
                   [mknode('name=', tag),
                    mknode('content=', node.arg, node.children)]))

    def gen_stdhtml(self, node, tag=None, **options):
        if tag is None:
            tag = node.tag
        node, attrs = node.split_attrs(tag)
        self.begin(tag, ' '.join(['%s=%r' % (key, val)
                                  for (key, val) in attrs]))
        if tag in self.mod._no_end_tag_elements:
            if node.arg:
                self.error(
                    'No enclosed text allowed for Html tag: %r.' % node.tag)
            self.no_children(node)
            self.indent -= self.indentstep
        else:
            node.arg_accept(self)
            self.end(tag)

    def get_encoded_text(self, node):
        # From a node's arg and children that are text or characters
        old_out = self.chg_out([])
        self.append(self.encode(node.arg))
        for ch in node.children:
            if ch.tag in ('text', 'char'):
                ch.accept(self)
            else:
                self.error('Only text and char allowed here, not %r.' %
                           ch.tag, ch)
        return ''.join(self.chg_out(old_out))

    def get_html(self):
        return ''.join(self.out)

    def no_children(self, node):
        if node.children:
            self.error('No children allowed for %r. Got children nodes = %r.' % (
                node.tag, node.children))

    def set_out(self, out):
        self.out = out
        self.extend = out.extend
        self.append = out.append

    def visit_author(self, node):
        self.gen_meta(node)

    def visit_block(self, node):
        self._visit_children(node)

    def visit_char(self, node):
        name = node.get_namearg()
        if name in self.mod.name2codepoint:
            name = '&%s;' % name
        else:
            if name[:2] == "0x":
                char = int(name[2:], 16)
            elif name.isdigit():
                char = int(name)
            else:
                self.error('No such character: %r.' % name, node)
            name = self.mod.codepoint2name.get(char)
            if name is None:
                name = '&#%d;' % char
            else:
                name = '&%s;' % name
        self.append(name)
        self._visit_children(node)

    def visit_col_width(self, node):
        self.append('<col width="%s" />' % node.arg)

    def visit_comment(self, node):
        return
        # self.append('<!-- %s -->'%node.arg)

    def visit_default(self, node):
        if node.tag in self.mod.stdhtml:
            if node.tag in self.mod._head_elements:
                self.head_nodes.append(node)
            else:
                self.gen_stdhtml(node)
        else:
            self.error('I don\'t know what to generate for the tag %r.' %
                       node.tag, node)

    def visit_define(self, node):
        name = self.encode_link_name(node.arg)
        self.begin('a', 'name=%r' % name)
        self._visit_children(node)
        self.end('a')

    def visit_document(self, node):
        self.indent = 2  # Known indentation of header to be generated later
        oldout = self.chg_out([])
        self._visit_children(node)

        # self.gen_generated_from_gsl()

        newout = self.chg_out(oldout)
        mknode = self.mod.node_of_taci
        lang = self.document_lang
        if not lang:
            lang = mknode('document_lang', 'en')

        self.indent = 0
        self.gen_document_header(lang, self.header_nodes)
        self.out.extend(newout)
        self.gen_document_trailer()

    def visit_document_lang(self, node):
        if self.document_lang is not None:
            self.error('Duplicate document lang directive.', node)
        self.document_lang = node

    def visit_document_title(self, node):
        self.header_nodes.append(self.mod.node_of_taci('title', node.arg))

    def visit_enumerate(self, node):
        self.begin('ol')
        for c in node.children:
            self.begin('li')
            c.accept(self)
            self.end('li')
        self.end('ol')

    def visit_exdefs(self, node):
        self.symplace = {}
        for ch in node.children:
            syms = [x.strip() for x in ch.arg.split(',')]
            for sym in syms:
                self.symplace[sym] = ch.tag

    def visit_generated_from_gsl(self, node):
        self.gen_generated_from_gsl()

    def visit_header(self, node):
        self.header_nodes.extend(node.children)

    def visit_itemize(self, node):
        self.begin('ul')
        for c in node.children:
            self.begin('li')
            c.accept(self)
            self.end('li')
        self.end('ul')

    def visit_link_to_extern(self, node):
        name = node.arg
        docname = node.children[0].arg
        children = node.children[1:]
        uri = '%s.html#%s' % (docname, self.encode_link_name(name))
        self.begin('a', 'href=%r' % uri)
        if not children:
            self.append(self.encode(name))
        else:
            for ch in children:
                ch.accept(self)
        self.end('a')

    def visit_link_to_local(self, node):
        name = node.arg
        uri = '#%s' % self.encode_link_name(name)
        self.begin('a', 'href=%r' % uri)
        if not node.children:
            self.append(self.encode(name))
        else:
            self._visit_children(node)
        self.end('a')

    def visit_link_to_unresolved(self, node):
        name = node.arg
        self.begin('em')
        if not node.children:
            self.append(self.encode(name))
        else:
            self._visit_children(node)
        self.end('em')

    def visit_literal_block(self, node):
        self.gen_stdhtml(node, 'pre')

    def visit_man_page_mode(self, node):
        self._visit_children(node)

    def visit_meta(self, node):
        self.document_metas.append(node)

    def visit_spc_colonkind(self, node):
        # self.append('&nbsp;<strong>:</strong>&nbsp;')
        # self.append('&nbsp;<code>:</code>&nbsp;')
        self.append('<code>:</code>&nbsp;')

    def visit_spc_mapsto(self, node):
        self.append(' <strong>-></strong> ')

    def visit_string(self, node):
        self._visit_children(node)

    def visit_symbol(self, node):
        self.visit_text(node)

    def visit_text(self, node):
        text = self.encode(node.arg)
        if len(text) > 80 or '\n' in text:
            self.append('\n')
        self.append(text)
        self._visit_children(node)

    def visit_to_document_only(self, node):
        self._visit_children(node)

    def visit_to_html_only(self, node):
        self._visit_children(node)

    def visit_to_tester_only(self, node):
        pass

    def visit_valid_html40(self, node):
        self.valid_html40 = node

        node, attrs = self.valid_html40.split_attrs(attrdict=True)
        # XXX check allowed attrs but in a GENERAL way
        # Code taken from validator.w3.org
        self.append("""\
    <a href="http://validator.w3.org/check?uri=referer"><img
        src="%s"
        alt="Valid HTML 4.0 Strict" height="31" width="88" /></a>
""" % attrs.get('src', 'http://www.w3.org/Icons/valid-html40'))

    def visit_with(self, node):
        pass

    def visit_word(self, node):
        self._visit_children(node)


class _GLUECLAMP_:
    _imports_ = (
        '_parent:SpecNodes',
        '_parent.SpecNodes:node_of_taci',
        '_parent.Gsml:is_not_ascii',
        '_parent.Main:tgt_prefix',
        '_parent.Main:ReportedError',
        '_root.htmlentitydefs:name2codepoint',
        '_root.htmlentitydefs:codepoint2name',
        '_root:re',
        '_root:time',
    )

    _chgable_ = ('tag_uppercase_name_chars',)

    # Set to make upper-case name characters tagged to make sure
    # no names in a file differ only in case as stated in HTML spec.
    # I believe this doesn't matter in practice in contemporary browsers,
    # since references are also said to be case sensitive!
    # -- I can't be bothered to solve this better now. See also Notes Aug 12 2005.
    tag_uppercase_name_chars = 0

    _html3_2 = (
        'a', 'address', 'area',
        'b', 'base', 'big', 'blockquote', 'body', 'br',
        'caption', 'center', 'cite', 'code',
        'dfn', 'dt', 'dl', 'dd', 'div',
        'em', 'form',
        'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'html',
        'i', 'img', 'input', 'kbd',
        'li',
        'ol', 'option',
        'p', 'param', 'pre',
        'samp', 'select', 'small', 'strong', 'style', 'sub', 'sup',
        'table', 'td', 'textarea', 'th', 'thead', 'title', 'tr', 'tt',
        'ul',
        'var')

    # Included in Html 3.2 but 'deprecated' in Html 4.0
    _html4_0_deprecated = (
        'applet', 'basefont', 'dir', 'font', 'isindex',
        'strike', 'u',
    )

    # Included in 3.2, not depreciated in 4.0 but one may want to avoid them
    _html_avoid = (
        'script',
    )
    _html4_0 = (
        'abbr', 'acronym',
        'bdo', 'button',
        'col', 'colgroup',
        'del',
        'fieldset', 'frame', 'frameset',
        'iframe', 'ins',
        'label', 'legend',
        'noframes', 'noscript',
        'object', 'optgroup',
        'q', 's', 'span',
        'tbody', 'tfoot', 'thead')

    _head_elements = (
        'base', 'isindex', 'link', 'meta', 'script', 'style', 'title'
    )

    # The ones that can have no end tag
    # xxx are there more -style etc- look it up!

    _no_end_tag_elements = (
        # Header elmts
        'meta', 'link',
        # Other
        'img',
        # CAN have end tag? but never has. This will self-close to generate valid XHTML.
        'hr',
    )

    # The ones that we may generate line-break before
    # and hope it will not affect the insertion of spaces in rendering.

    _line_break_allowed = (
        'html', 'head', 'body', 'frameset',
        # Head Elements
    ) + _head_elements + (
        # Generic Block-level Elements
        'address', 'blockquote', 'center', 'del', 'div',
        'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'ins', 'isindex', 'noscript', 'p', 'pre',
        # Lists
        'dir', 'dl', 'dt', 'dd', 'li', 'menu', 'ol', 'ul',
        # Tables
        'table', 'caption', 'colgroup', 'col', 'thead', 'tfoot', 'tbody', 'tr', 'td', 'th',
        # Forms
        'form', 'button', 'fieldset', 'legend', 'input', 'label',
        'select', 'optgroup', 'option', 'textarea'
    )

    # The attributes allowed in META elements
    meta_attributes = ('name', 'http-equiv', 'content',
                       'scheme', 'lang', 'dir')

    # This returns a function checking if a character is allowed to be used
    # as the first character in a NAME or ID attribute.
    #   (I don't think this is the same as .isalpha() with unicode.)

    def _get_is_name_starter_char(self):
        return self.re.compile(r"[A-Za-z]").match

    # This returns a function checking if a character is allowed to be used
    # after the first character in a NAME or ID attribute.

    def _get_is_name_follower_char(self):
        return self.re.compile(r"[A-Za-z0-9\-_:\.]").match

    # A set of the ones we generate directly.
    # This includes the ones from html 3.2 and
    # I have also included the deprecated and the 4.0 only

    def _get_stdhtml(self):
        sh = {}
        for x in self._html3_2 + self._html4_0_deprecated + self._html4_0:
            sh[x] = 1
        return sh

    def _get_line_break_allowed(self):
        sh = {}
        for x in self._line_break_allowed:
            sh[x] = 1
        return sh

    def doc2filer(self, doc, node, name, dir, opts, IO):
        text = self.doc2text(doc, node)
        path = IO.path.join(dir, '%s.html' % name)
        node = self.node_of_taci('write_file', path, [
                                 self.node_of_taci('text', text)])
        return node

    def doc2text(self, doc, node):
        d2h = Node2XHTML(self, node, doc.env.error)
        return d2h.get_html()

    def node2file(self, node, file):
        text = self.node2text(node)
        with open(file, 'w') as f:
            f.write(text)

    def node2text(self, node):
        text = Node2XHTML(self, node).get_html()
        return text

    # Adapted from html4css1.py in docutils

    def encode(self, text):
        """Encode special characters in `text` & return."""
        # @@@ A codec to do these and all other HTML entities would be nice.
        text = text.replace("&", "&amp;")
        text = text.replace("<", "&lt;")
        text = text.replace('"', "&quot;")
        text = text.replace(">", "&gt;")
        text = text.replace("@", "&#64;")  # may thwart some address harvesters
        return text

    # Encode a name according to HTML spec. See also Notes Aug 12 2005.
    # From wdghtml40/values.html#cdata :
    # Attribute values of type ID and NAME must begin with a letter in the
    # range A-Z or a-z and may be followed by letters (A-Za-z), digits
    # (0-9), hyphens ("-"), underscores ("_"), colons (":"), and periods
    # ("."). These values are case-sensitive.

    def encode_name(self, name):
        is_name_follower_char = self.is_name_follower_char
        ns = []
        append = ns.append
        upperstate = 0

        ch = name[:1]
        if ch == 'z' or not self.is_name_starter_char(ch):
            append('z')
            if ch == 'z':
                append('z')
        for ch in name:
            if ch == '-' or not is_name_follower_char(ch):
                if upperstate:
                    append('-')
                    upperstate = 0
                append('-')
                if ch != '-':
                    append('%d' % ord(ch))
                append('-')
            elif ch.isupper() and self.tag_uppercase_name_chars:
                if not upperstate:
                    append('-')
                    upperstate = 1
                append(ch)
            else:
                if upperstate:
                    append('-')
                    upperstate = 0
                append(ch)
        if upperstate:
            append('-')
        return ''.join(ns)



from guppy.heapy.test import support
import inspect
import unittest


class TestCase(support.TestCase):
    def setUp(self):
        support.TestCase.setUp(self)
        self.Path = self.heapy.Path

    def chkrel(self, src, dst, relstr=None, clas=None):
        rel = self.relation(src, dst)
        if clas is not None:
            self.assertTrue(isinstance(rel, clas))
        if relstr is None:
            print(rel)
        else:
            sr = str(rel)
            if sr.startswith('<') and not relstr.startswith('<'):
                self.assertTrue(sr.endswith('>'))
                sr = sr[1:-1].split(',')
                self.assertTrue(relstr in sr)
            else:
                self.aseq(sr, relstr)

    def chkrelattr(self, src, *attrs):
        for attr in attrs:
            self.chkrel(src, getattr(src, attr), '%s.'+attr)

    def chkpath(self, src, dst, expect=None):
        rel = self.shpaths(dst, src)
        if expect is None:
            print(rel)
        else:
            li = rel.aslist()
            if len(li) == 1:
                li = li[0]
            self.aseq(str(li), str(expect), -1)

    def relation(self, src, dst):
        return self.Path.relation(src, dst)

    def shpaths(self, dst, src=None, *args, **kwds):
        # return self.Path.shpaths(dst, src, *args, **kwds)

        dst = self.iso(dst)
        if src is not None:
            src = self.iso(src)
        return dst.get_shpaths(src, *args, **kwds)


class RelationTestCase(TestCase):
    # Test relations from standard types and some simple paths

    def test_cell_relation(self):
        cellvalue = []

        def f():
            return cellvalue
        self.chkrel(f.__closure__[0], cellvalue, '%s.cell_contents')

    def test_code_relation(self):
        def f():
            a = 3
            return self, a
        co = f.__code__
        # xxx brittle test but catches a bug
        self.chkpath(co, 3, '%s.co_consts[1]')
        # commented in notes Sep 27 2004
        self.chkrelattr(co, 'co_code', 'co_consts', 'co_names', 'co_varnames',
                        'co_freevars', 'co_cellvars', 'co_filename', 'co_name',
                        'co_lnotab')


    def test_dict_relation(self):
        k1 = 'k1'
        k2 = 'k2'
        v1 = 'v1'
        v2 = 'v2'
        k3 = tuple(range(100))
        v3 = tuple(range(100, 200))
        x = {k1: v1, k2: v2, k3: v3}
        self.chkrel(x, v1, "%s['k1']")
        self.chkrel(x, v2, "%s['k2']")
        self.chkrel(x, v3, "%s[(0, 1, 2, 3, 4, 5, ...)]")
        ks = [str(self.relation(x, k1)),
              str(self.relation(x, k2)),
              str(self.relation(x, k3))]
        ks.sort()
        self.aseq(ks, ['%s.keys()[0]', '%s.keys()[1]', '%s.keys()[2]'])

    def test_dictproxy_relation(self):
        v1 = 'v1'

        class T(object):
            k1 = v1
        x = T.__dict__
        self.chkpath(x, v1, "%s->mapping['k1']")
        self.chkrel(x, v1, "%s['k1']")

    def test_frame_relation(self):
        f = inspect.currentframe()
        f.f_trace = lambda: None
        self.chkrelattr(f, 'f_back', 'f_builtins', 'f_code', 'f_globals',
                        'f_locals', 'f_trace')

        a = []
        # The representation of local variables is how they may be accessed
        # - not how they are really stored.
        # xxx this may be confusing/lack information?
        # The information is available in the relation object class,
        # it is just not represented with str()...
        self.chkrel(f, a, "%s.f_locals['a']", clas=self.Path.R_LOCAL_VAR)

        x = []
        z = []

        def func(x, y=3):
            frame = inspect.currentframe()
            return self, frame, z
        _, frame, __ = func(0)
        del _, __
        self.chkrel(frame, self, "%s.f_locals ['self']", clas=self.Path.R_CELL)

        self.chkrel(f, x, "%s.f_locals['x']", clas=self.Path.R_LOCAL_VAR)
        self.chkrel(f, z, "%s.f_locals ['z']", clas=self.Path.R_CELL)
        # self becomes both a local var and a cell var, since it is an argument.
        # FIXME: But not in py3... it seems
        # self.chkrel(f, self, "<%s.f_locals['self'],%s.f_locals ['self']>")

        # Stack variables doesn't work (Because ceval.c doesn't update
        # the f_stacktop index.) so the corresponding part of frame_relate is not tested.

    def test_function_relation(self):
        def f(x, y=3):
            return self
        f.a = []

        self.chkrelattr(f, '__code__', '__globals__', '__defaults__',
                        '__closure__', '__doc__', '__name__', '__dict__',
                        'a')

    def test_instancemethod_relation(self):
        class T:
            def f(x):
                pass
        t = T()
        self.chkrelattr(t.f, '__func__', '__self__')

    def test_list_relation(self):
        v1 = 'v1'
        v2 = 'v2'
        v3 = list(range(100, 200))
        x = [v1, v2, v3]
        self.chkrel(x, v1, '%s[0]')
        self.chkrel(x, v2, '%s[1]')
        self.chkrel(x, v3, '%s[2]')

    def test_meth_relation(self):
        x = []
        self.chkrel(x.append, x, '%s.__self__')

    def test_module_relation(self):
        self.chkrelattr(unittest, '__dict__', 'TestCase')

    def test_nodegraph_relation(self):
        a = 0
        b = 1
        rl = [a, b]
        rg = self.heapy.heapyc.NodeGraph([(a, rl), (b, rl)])
        self.chkrel(rg, a, '%s->edges[0].src')
        self.chkrel(rg, b, '%s->edges[1].src')
        self.chkrel(rg, rl, '<%s->edges[0].tgt,%s->edges[1].tgt>')
        self.chkpath(rg, a, '%s->edges[0].src')
        self.chkpath(rg, rl, ['%s->edges[0].tgt', '%s->edges[1].tgt'])

    def test_nodeset_relation(self):
        from guppy.sets import immnodeset, mutnodeset
        x = ['a']
        for s in (immnodeset(x), mutnodeset(x)):
            for i in range(len(x)):
                self.chkrel(s, x[i], 'list(%%s)[%s]' % i)

    def test_object_relation(self):
        class T(object):
            __slots__ = 'a', 'b'
        t = T()
        a = []
        t.a = a
        b = []
        t.b = b
        # self.chkrel(t, T, 'type(%s)')
        self.chkrel(t, T, '%s->ob_type')
        self.chkrelattr(t, 'a', 'b')
        # We shouldn't have a __dict__ here - just make sure this is the case
        self.assertRaises(AttributeError, lambda: t.__dict__)

        class U(T):
            pass
        u = U()
        u.a = a
        self.chkpath(u, T, "%s->ob_type.__base__")
        self.chkrel(u, a, '%s.a')

        c = []
        u.c = c
        self.chkrel(u, c, '%s.c')
        self.chkrel(u, u.__dict__, '%s.__dict__')

        class V(U):
            pass
        v = V()
        v.c = c
        self.chkrelattr(v, '__dict__')

        class W(V):
            __slots__ = 'c', 'd', 'b'
            pass
        w = W()
        w.a = a
        w.b = b
        w.c = c
        w.d = []
        w.e = []
        self.chkrelattr(w, '__dict__', 'a', 'b', 'c', 'd', 'e')
        self.chkpath(w, w.a, '%s.a')
        self.chkpath(w, w.b, '%s.b')
        self.chkpath(w, w.c, '%s.c')
        self.chkpath(w, w.d, '%s.d')
        self.chkpath(w, w.e, "%s.__dict__['e']")

        class R(object):
            rvar = []

        class S(R, T):
            svar = []

        s = S()
        s.a = a
        s.b = b
        s.c = c
        self.chkrelattr(s, '__dict__', 'a', 'b', 'c')
        self.chkpath(s, s.a, '%s.a')
        self.chkpath(s, s.b, '%s.b')
        self.chkpath(s, s.c, "%s.__dict__['c']")

        # Class variables are not directly related- should they be that?
        # Possibly, but the compression could as well be done in Python.
        # We just check that we can get the path.
        self.chkpath(s, s.svar, "%s->ob_type.__dict__['svar']")
        self.chkpath(s, s.rvar, ["%s->ob_type.__bases__[0].__dict__['rvar']",
                                 "%s->ob_type.__mro__[1].__dict__['rvar']"])
        self.chkpath(s, s.__slots__,
                     "%s->ob_type.__base__.__dict__['__slots__']")

    def test_traceback_relation(self):
        import sys
        try:
            def g():
                1/0
            g()
        except ZeroDivisionError:
            type, value, traceback = sys.exc_info()
        self.chkrelattr(traceback, 'tb_next', 'tb_frame')

    def test_tuple_relation(self):
        v1 = 'v1'
        v2 = 'v2'
        v3 = list(range(100, 200))
        x = (v1, v2, v3)
        self.chkrel(x, v1, '%s[0]')
        self.chkrel(x, v2, '%s[1]')
        self.chkrel(x, v3, '%s[2]')

    def test_type_relation(self):
        name = 'T'
        base = object
        bases = (base,)
        dict = {'__slots__': ('a', 'b')}
        T = type(name, bases, dict)
        # tp_dict can't be directly tested since .__dict__ returns a proxy
        # and the dict passed is not used directly.
        # We test it indirectly by getting a path through it.
        self.chkpath(T, T.a, "%s.__dict__['a']")
        # The C-struct __slots__ field can't be tested directly
        # This just tests the ordinary attribute
        self.chkpath(T, T.__slots__, "%s.__dict__['__slots__']")
        self.chkrelattr(T, '__mro__', '__base__', '__bases__')
        # tp_cache and tp_subclasses can also not be tested directly

        # Inheritance is tested via test_object_relation()


class RootTestCase(TestCase):
    def test_1(self):
        import sys
        import builtins
        root = self.View.root
        # Interpreter attributes

        rel = str(self.relation(root, sys.modules))

        self.assertTrue(eval(rel % 'root') is sys.modules)
        self.aseq(rel, '%s.i0_modules')

        rel = str(self.relation(root, sys.__dict__))
        self.assertTrue(eval(rel % 'root') is sys.__dict__)
        self.aseq(rel, '%s.i0_sysdict')

        rel = str(self.relation(root, builtins.__dict__))
        self.assertTrue(eval(rel % 'root') is builtins.__dict__)
        self.aseq(rel, '%s.i0_builtins')

        for name in "codec_search_path", "codec_search_cache", "codec_error_registry":
            attr = "i0_%s" % name
            rel = str(self.relation(root, getattr(root, attr)))
            self.aseq(rel, '%%s.%s' % attr)

        # Thread attributes

        try:
            1/0
        except ZeroDivisionError:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            for name in 'exc_type', 'exc_value', 'exc_traceback':
                rel = str(self.relation(root, eval(name)))
                self.asis(eval(rel % 'root'), eval(name))

            # There are more, untested, attributes, but the code is farily regular...
            # More complication is to do with frames which I concentrate on for now.

            # We need to find out what level we are at - count to lowest frame
            level = 0
            frame = exc_traceback.tb_frame

            while frame.f_back:
                frame = frame.f_back
                level += 1
            rel = str(self.relation(root, frame))
            self.assertTrue(rel.endswith('_f0'))
            rel = str(self.relation(root, exc_traceback.tb_frame))
            self.asis(eval(rel % 'root'), exc_traceback.tb_frame)
            self.assertTrue(rel.endswith('_f%d' % level))

    def test_thread(self):
        try:
            import _thread
        except ImportError:
            print('threading not enabled - skipping test')
            return

        root = self.View.root

        def task(self):
            import sys
            try:
                1/0
            except ZeroDivisionError:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                self.exc_traceback = exc_traceback
                self.sync = 1
                while self.sync:
                    pass
                self.sync = 1

        self.sync = 0
        _thread.start_new_thread(task, (self,))
        while not self.sync:
            pass
        exc_traceback = self.exc_traceback
        rel = str(self.relation(root, exc_traceback))
        self.asis(eval(rel % 'root'), exc_traceback)
        self.sync = 0
        while not self.sync:
            pass

        def task(self):
            self.test_1()
            self.sync = 1
        self.sync = 0
        _thread.start_new_thread(task, (self,))
        while not self.sync:
            pass

    def test_secondary_interpreter(self):
        import sys
        if sys.version_info >= (3, 9):
            print('multi-interpreter not supported past Python 3.9')
            return

        try:
            import _thread
        except ImportError:
            print('threading not enabled - skipping test')
            return

        import_remote = """\
import sys
import _thread
import time

def task():
    time.sleep(1)

    self.sysdict = sys.__dict__
    self.sync = 1

    while self.sync:
        pass

_thread.start_new_thread(task, ())
"""

        self.sync = 0
        thid = self.heapy.heapyc.interpreter(import_remote, {'self': self})

        root = self.View.root

        import sys
        sysdict = sys.__dict__
        rel = str(self.relation(root, sysdict))
        self.aseq(rel, '%s.i0_sysdict')

        while not self.sync:
            pass

        rel = str(self.relation(root, self.sysdict))
        self.aseq(rel, '%s.i1_sysdict')

        self.sync = 0


class PathTestCase(TestCase):
    def makegraph(self, width, length):
        # Generate a structure which will yield a high number
        # of shortest paths.
        # Returns a pair src, dst which are connected via a noncyclic graph
        # with many edges.
        # The length of each path (all shortest), number of edges will be length
        # The number of nodes will be 2 + width * (length - 1)
        # The number of paths will be
        #       width ** length, if width >= 1 and length >= 1

        dst = []
        ls = []
        for i in range(width):
            ls.append([dst])
        ls = [dst] * width
        for i in range(length-1):
            xs = []
            for j in range(width):
                ys = []
                xs.append(ys)
                for k in range(width):
                    ys.append(ls[k])
            ls = xs
        src = ls
        return src, dst

    def chkgraph(self, width, length, expect=None):
        src, dst = self.makegraph(width, length)
        self.chkpath(src, dst, expect)

    def test_path(self):
        dst = 'dst'
        self.chkpath([dst], dst, '%s[0]')
        self.chkpath([[], dst], dst, '%s[1]')
        self.chkpath([dst, dst], dst, "['%s[0]', '%s[1]']")
        self.chkpath([[dst, 0], dst, [dst, 2]], dst, "%s[1]")
        self.chkpath([[dst, 0], [dst, 2]], dst, "['%s[0][0]', '%s[1][0]']")

        src, dst = self.makegraph(1, 1)

        self.chkgraph(1, 1, '%s[0]')
        self.chkgraph(1, 2, '%s[0][0]')
        self.chkgraph(2, 1, ['%s[0]', '%s[1]'])
        self.chkgraph(3, 2, ['%s[0][0]', '%s[0][1]',     '%s[0][2]',
                             '%s[1][0]', '%s[1][1]',     '%s[1][2]',
                             '%s[2][0]', '%s[2][1]',     '%s[2][2]'])

    def test_numpaths(self):
        for (width, length) in [(2, 1), (7, 3), (3, 7), (10, 20)]:
            src, dst = self.makegraph(width, length)
            p = self.shpaths(dst, src)
            self.aseq(p.numpaths, width**length)

    def test_iter(self):
        src, dst = self.makegraph(2, 2)
        p = self.shpaths(dst, src)
        it = iter(p)
        ss = []
        for i in it:
            ss.append(str(i))
        ss.sort()
        self.aseq(ss, ['%s[0][0]', '%s[0][1]', '%s[1][0]', '%s[1][1]'])

        # Check that we can get some of the first values from the iterator
        # of a graph with an astronomical number of paths.

        width = 11
        length = 13
        numpaths = 20
        src, dst = self.makegraph(width, length)
        p = self.shpaths(dst, src)
        it = iter(p)
        for i in range(numpaths):
            path = next(it)
            sp = str(path)
            div, mod = divmod(i, width)
            self.aseq(sp, '%s'+'[0]'*(length-2)+'[%d][%d]' % (div, mod))

        # Check that the iterator works even if the graph initially
        # would yield astronomical numbers of dead ends.
        # (The initial algorithm took astronomically long time.)

        osrc = src

        src, dst = self.makegraph(width, length)
        src[0] = osrc
        p = self.shpaths(dst, src)
        it = iter(p)
        for i in range(numpaths):
            path = next(it)
            sp = str(path)
            div, mod = divmod(i, width)
            self.aseq(sp, '%s[1]'+'[0]'*(length-3)+'[%d][%d]' % (div, mod))

        # Test iterating with a negative start and a large positive start

        numfromend = width / 2
        for it in [p.iter(-numfromend), p.iter(p.numpaths-numfromend)]:
            for i, path in enumerate(it):
                sp = str(path)
                self.aseq(sp, '%s'+('[%d]' % (width-1)) *
                          (length-1)+'[%d]' % (width-numfromend+i))

        # Test iterating with start and stop

        start = 5
        stop = 25
        i = start
        for path in p.iter(start, stop):
            sp = str(path)
            div, mod = divmod(i, width)
            self.aseq(sp, '%s[1]'+'[0]'*(length-3)+'[%d][%d]' % (div, mod))
            self.aseq(path.index, i)
            i += 1
        self.aseq(i, stop)

    def test_str(self):
        # Make sure large number of paths will yield reasonable representations
        width = 11
        length = 4
        src, dst = self.makegraph(width, length)
        p = self.shpaths(dst, src)
        p.maxpaths = 1
        self.aseq(str(p), " 0: Src[0][0][0][0]\n<... 14640 more paths ...>")
        p.maxpaths = 2
        self.aseq(
            str(p), " 0: Src[0][0][0][0]\n 1: Src[0][0][0][1]\n<... 14639 more paths ...>")

    def test_printing(self):
        # Test the pretty-printing and moreing methods
        from io import StringIO
        output = StringIO()
        self.Path.output = output
        width = 11
        length = 4
        src, dst = self.makegraph(width, length)
        p = self.shpaths(dst, src)
        p.maxpaths = 2
        self.aseq(str(p), """\
 0: Src[0][0][0][0]
 1: Src[0][0][0][1]
<... 14639 more paths ...>""")
        self.aseq(str(p.more), """\
 2: Src[0][0][0][2]
 3: Src[0][0][0][3]
<... 14637 more paths ...>""")

    def test_subscript(self):
        # Test subscripting
        width = 3
        length = 40
        src, dst = self.makegraph(width, length)
        p = self.shpaths(dst, src)
        np = width**length
        self.aseq(np, p.numpaths)
        # p[0].pp(p.output)
        self.aseq(str(p[0]), '%s'+'[0]'*length)
        self.aseq(str(p[-np]), '%s'+'[0]'*length)
        self.aseq(str(p[width-1]), '%s'+'[0]'*(length-1) + '[%d]' % (width-1))
        self.aseq(str(p[width]), '%s'+'[0]'*(length-2) + '[1][0]')
        self.aseq(str(p[width+1]), '%s'+'[0]'*(length-2) + '[1][1]')
        self.aseq(str(p[np-1]), '%s'+('[%d]' % (width-1))*length)
        self.aseq(str(p[-1]), '%s'+('[%d]' % (width-1))*length)
        self.assertRaises(IndexError, lambda: p[np])
        self.assertRaises(IndexError, lambda: p[-np-1])


class MultiTestCase(TestCase):
    def test_pp(self):
        # Test printing of multi relations
        iso = self.iso
        dst = [[], []]
        src = iso(dst[:]*2)
        dst = [iso(x) for x in dst]
        p = self.Path.shpgraph(dst, src)
        self.aseq(str(p), """\
--- Dst[0] ---
 0: Src[0]
 1: Src[2]
--- Dst[1] ---
 0: Src[1]
 1: Src[3]""")
        p = self.Path.shpgraph(dst, src, srcname='A', dstname='B')
        self.aseq(str(p), """\
--- B[0] ---
 0: A[0]
 1: A[2]
--- B[1] ---
 0: A[1]
 1: A[3]""")


class AvoidTestCase(TestCase):
    def test_1(self):
        # Test that we can find new paths by avoiding edges
        # selected from previously found paths.
        # First we generate a graph with paths of various lengths...

        src = ['src']
        a = src
        for i in range(3):
            b = ['b%d' % i]
            c = ['c%d' % i, b]
            a.append(b)
            a.append(c)
            a = b
        dst = a
        p = self.shpaths(dst, src)

        for avoid, result in [
            ([],        '%s[1][1][1]'),
            ([0],       '%s[2][1][1][1]'),
            ([1],       '%s[1][2][1][1]'),
            ([2],       '%s[1][1][2][1]'),
            ([0, 1],    '%s[2][1][2][1][1]'),
            ([1, 2],    '%s[1][2][1][2][1]'),
            #           ([1, -1],   '%s[1][2][1][2][1]'),
            ([0, 2],    '%s[2][1][1][2][1]'),
            ([0, 1, 2], '%s[2][1][2][1][2][1]'),
            ([2, 1, 0], '%s[2][1][2][1][2][1]'),

        ]:
            result = result % ' 0: Src'
            # Find new path by avoiding edges from the original path
            q = self.shpaths(dst, src, avoid_edges=p.edges_at(*avoid))
            self.aseq(str(q), result)
            # Find the same path but via a direct method
            q = p.copy_but_avoid_edges_at_levels(*avoid)
            self.aseq(str(q), result)
            # The same, but via a shorter method name
            q = p.avoided(*avoid)
            self.aseq(str(q), result)

        # Test that the avoided set is carried on to copies

        q = p.avoided(0).avoided(2)
        self.aseq(str(q), ' 0: Src[2][1][2][1][1]')


class NewTestCase(TestCase):
    def test_1(self):
        import sys
        o = self.python.io.StringIO()
        iso = self.iso
        x = iso(sys.__dict__)
        print(x.shpaths, file=o)
        # This used to include a path via parameter avoid_edges
        # which was confusing
        print(x.shpaths.avoided(0), file=o)

        # repr() used to be quite useless. I have it now defined as .pp(),
        # but without trailin newline.

        print(repr(x.shpaths), file=o)
        print(repr(x.shpaths), file=o)

        # The shpaths object could sometimes disturb a shpath calculation
        # because dst was free in it.

        x = []
        y = [[[x]]]

        sp = iso(x).get_shpaths(iso(y))
        print(sp, file=o)

        y.append(sp)
        print(iso(x).get_shpaths(iso(y)), file=o)

        # Test that the shortest paths to a set of objects, is the shortest
        # paths to those that can be reached by the shortest paths, only

        x = []
        y = [x]
        z = [y]

        print(iso(x, y).get_shpaths(iso(z)), file=o)

        # Test that we can relate objects that inherits from a class and object
        # (Used to segfault)

        class C:
            pass

        class O(C, object):
            __slots__ = 'x',

        ob = O()
        ob.x = x

        print(iso(x).get_shpaths(iso(ob)), file=o)

        # Test that generalization to a set of sources makes some sense
        # The shortest paths are from the closest sources

        # Hack to make a constant address rendering, for test comparison.
        # This doesn't change anything permanently.
        # XXX come up with an official way to do this.
        summary_str = self.heapy.UniSet.summary_str
        def str_address(x): return '<address>'
        str_address._idpart_header = getattr(
            summary_str.str_address, '_idpart_header', None)
        str_address._idpart_sortrender = getattr(
            summary_str.str_address, '_idpart_sortrender', None)
        summary_str.str_address = str_address

        S = iso()
        shp = iso(x).get_shpaths(iso(y, z))
        print(shp, file=o)
        print(repr(shp), file=o)
        for p in shp:
            S = S ^ p.src
        self.aseq(S, iso(y))

        shp = iso(x).get_shpaths(iso(ob, y, z))
        print(str(shp), file=o)
        print(repr(shp), file=o)
        S = iso()
        for i, p in enumerate(shp):
            S = S ^ p.src
            self.aseq(p.src, shp[i].src)
        self.aseq(S, iso(ob, y))

        # Test that the iter can be restarted
        # even after multiple sources handling was added

        it = iter(shp)
        a = list(it)
        it.isatend = 0
        b = list(it)
        self.aseq(str(a), str(b))

        self.aseq(o.getvalue(), """\
 0: hpy().Root.i0_sysdict
 0: Src.i0_modules['sys'].__dict__
 0: hpy().Root.i0_sysdict
 0: hpy().Root.i0_sysdict
 0: Src[0][0][0]
 0: Src[0][0][0]
 0: Src[0]
 0: Src.x
 0: <1 list: <address>*1>[0]
 0: <1 list: <address>*1>[0]
 0: <1 list: <address>*1>[0]
 1: <1 __main__.O: <address>>.x
 0: <1 list: <address>*1>[0]
 1: <1 __main__.O: <address>>.x
""".replace('__main__', self.__module__))

    def test_2(self):
        # To assist interactivity,
        # the more attribute is defined to return an object which
        # the repr() of gives more lines; and has a similar more attribute.
        # Testing this functionality here.

        o = self.python.io.StringIO()
        iso = self.iso
        dst = []
        src = [dst]*20
        print(repr(iso(dst).get_shpaths(iso(src))), file=o)
        print(repr(iso(dst).get_shpaths(iso(src)).more), file=o)
        p = iso(dst).get_shpaths(iso(src))
        print(repr(p.more), file=o)

        self.aseq(o.getvalue(), """\
 0: Src[0]
 1: Src[1]
 2: Src[2]
 3: Src[3]
 4: Src[4]
 5: Src[5]
 6: Src[6]
 7: Src[7]
 8: Src[8]
 9: Src[9]
<... 10 more paths ...>
10: Src[10]
11: Src[11]
12: Src[12]
13: Src[13]
14: Src[14]
15: Src[15]
16: Src[16]
17: Src[17]
18: Src[18]
19: Src[19]
10: Src[10]
11: Src[11]
12: Src[12]
13: Src[13]
14: Src[14]
15: Src[15]
16: Src[16]
17: Src[17]
18: Src[18]
19: Src[19]
""")

    def test_empty(self):
        # Test empty paths
        iso = self.iso
        dst = []
        self.assertTrue(len(list(iso(dst).get_shpaths(iso()))) == 0)

    def test_3(self):
        # Test that Edges is not included in the shortest path

        iso = self.iso
        dst = []
        shp = iso(dst).shpaths
        del dst
        self.assertTrue('Edges' not in str(shp.avoided(0)))

        dst = []

        src = [dst]

        shp = iso(dst).get_shpaths(iso(src))

        src[0] = shp

        dst = iso(dst)
        src = iso(src)

        self.assertTrue(dst.get_shpaths(src).numpaths == 0)

        # Test the sets attribute

        dst = []
        src = [dst]
        dst = iso(dst)
        src = iso(src)
        self.aseq(dst.get_shpaths(src).sets, (src, dst))

        # Test that srs doesn't disturb the path calculation

        class C:
            pass

        c = C()
        cd = iso(c.__dict__)

        p = cd.shpaths
        repr(p)

        del c
        q = cd.shpaths
        self.aseq(repr(q).strip(), "")

        del p, q

        # Test that internals of ShortestPath are hidden in general
        # (via hiding_tag), to consistent result when used interactively,
        # as commented on in notes.txt per Nov 30 2004.

        dst = []
        src = [[[[dst]]]]

        d = iso(dst)
        s = iso(src)

        p = d.get_shpaths(s)
        self.aseq(str(p), " 0: Src[0][0][0][0]")
        src.append(p)
        p._XX_ = dst    # A shorter path, but it should be hidden
        self.aseq(str(d.get_shpaths(s)), " 0: Src[0][0][0][0]")

        # Test what .more prints finally

        self.aseq(str(p.more), '<No more paths>')

        # Test that .top is idempotent

        self.asis(p.more.top.top, p)

    def test_4(self):
        # Test that if one extra path will be printed if there are
        # exactly 11 paths

        iso = self.iso
        o = self.python.io.StringIO()

        dst = []
        src = [dst] * 11

        shp = iso(dst).get_shpaths(iso(src))
        print(str(shp), file=o)
        self.aseq(o.getvalue(), """\
 0: Src[0]
 1: Src[1]
 2: Src[2]
 3: Src[3]
 4: Src[4]
 5: Src[5]
 6: Src[6]
 7: Src[7]
 8: Src[8]
 9: Src[9]
10: Src[10]
""")


def run_test(case, debug=0):
    support.run_unittest(case, debug)


def test_main(debug=0):
    run_test(NewTestCase, debug)
    run_test(RelationTestCase, debug)
    run_test(RootTestCase, debug)
    run_test(PathTestCase, debug)
    run_test(MultiTestCase, debug)
    run_test(AvoidTestCase, debug)


if __name__ == "__main__":
    test_main()



# Run an xterm on current process or a forked process

# Adapted from pty.py in Python 1.5.2 distribution.
# The pty.fork() couldnt be used because it didn't return
# the pty name needed by xterm
# I couldnt import pty.py to use master_open because it didn't find termios.

import os
import sys
import fcntl

# We couldnt find termios

STDIN_FILENO, STDOUT_FILENO, STDERR_FILENO = 0, 1, 2

# Open pty master.  Returns (master_fd, tty_name).  SGI and Linux/BSD version.
# Copied from pty.py from Python 1.5.2. /SN


def master_open():
    try:
        import sgi
    except ImportError:
        pass
    else:
        try:
            tty_name, master_fd = sgi._getpty(fcntl.O_RDWR, 0o666, 0)
        except IOError as msg:
            raise os.error(msg)
        return master_fd, tty_name
    for x in 'pqrstuvwxyzPQRST':
        for y in '0123456789abcdef':
            pty_name = '/dev/pty' + x + y
            try:
                fd = os.open(pty_name, fcntl.O_RDWR)
            except os.error:
                continue
            return (fd, '/dev/tty' + x + y)
    raise os.error('out of pty devices')

# Open the pty slave.  Acquire the controlling terminal.
# Returns file descriptor.  Linux version.  (Should be universal? --Guido)
# Copied from pty.py from Python 1.5.2. /SN


def slave_open(tty_name):
    return os.open(tty_name, fcntl.O_RDWR)


def xterm(prog=None, options=''):
    master_fd, tty_name = master_open()
    pid = os.fork()
    if pid:
        # Acquire controlling terminal.
        slave_fd = slave_open(tty_name)

        # Slave becomes stdin/stdout/stderr of child.
        os.dup2(slave_fd, STDIN_FILENO)
        os.dup2(slave_fd, STDOUT_FILENO)
        os.dup2(slave_fd, STDERR_FILENO)
        if (slave_fd > STDERR_FILENO):
            os.close(slave_fd)
        os.close(master_fd)
        sys.stdin.readline()  # Throw away an init string from xterm
        if prog is not None:
            prog()
    else:
        os.setsid()
        cmd = 'xterm %s -S%s%d' % (options, tty_name[-2:], master_fd)
        os.system(cmd)
        #os.waitpid(pid, 0)
    return pid


def forkxterm(prog=None, options=''):
    pid = os.fork()
    if pid:
        return pid
    else:
        os.setsid()
        pid = xterm(prog, options)
        if not pid:
            os._exit(0)


def hello():
    print('hello')
    while 1:
        pass



import guppy.etc.Glue


class _GLUECLAMP_(guppy.etc.Glue.Interface):
    _preload_ = '_hiding_tag_',
    _chgable_ = ('reprefix', 'default_reprefix', 'gcobjs',
                 'relheap', 'relheapg', 'relheapu', '__doc__')
    _dir_ = (
        'Anything', 'Clodo', 'Id', 'Idset', 'Module',
        'Nothing', 'Prod', 'Rcs', 'Root', 'Size', 'Type', 'Unity',
        'Via', 'doc', 'findex', 'heap', 'heapu',
        'idset', 'iso', 'load', 'monitor', 'pb',
        'setref', 'test')

    _private_ = ('View', '_hiding_tag_', '_load_stat',  '_check_tracemalloc',
                 'ctime', 'default_reprefix', 'dumph', 'gcobjs', 'heapg',
                 'loadc', 'relheap', 'relheapg', 'relheapu', 'reprefix',
                 'setrelheap', 'setrelheapg', 'setrelheapu',
                 'tc_adapt', 'tc_repr', 'union', 'uniset_from_setcsatable',
                 'warnings', 'Stat'
                 )

    default_reprefix = 'hpy().'

    def _get_gcobjs(self):
        return self.Nothing

    def _get_relheap(self):
        return self.Nothing

    def _get_relheapg(self):
        return self.Nothing

    def _get_relheapu(self):
        return self.Nothing

    def _get_reprefix(self):
        # The name that this instance (or one with the same ._share)
        # has in the __main__ module, if any, or self.default_reprname otherwise.
        # Used for prefixing the result of repr() of various objects
        # so it becomes possible to evaluate it in a typical environment.
        import __main__
        for k, v in list(__main__.__dict__.items()):
            if (isinstance(v, self.__class__) and
                    getattr(v, '_share', None) is self._share):
                return '%s.' % k
        return self.default_reprefix

    def _get_Root(self):
        """Root: RootStateType

This attribute is a symbolic root containing attributes from which all
reachable objects in the heap can be reached. It is the only value (a
singleton) of its kind; see [1] for a description of its attributes.

References
        [0] heapy_Use.html#heapykinds.Use.Root
        [1] heapy_RootState.html#heapykinds.RootStateType"""

        return self.View.heapyc.RootState

    def __repr__(self):
        return """\
Top level interface to Heapy.
Use eg: %sdoc for more info on %s.""" % (
            self.reprefix, self.reprefix[:-1])

    def Ddir(self, opts=''):
        """\
        #OBSOLETE
$HP.dir(opts: str+])-> GuppyDir
$HP.dir(opts: str+]).<attribute> -> GuppyDoc

A replacement for the builtin function dir(), providing a listing of
public attributes for Heapy objects. It also has an attribute for each
item in the listing, for example:

>>> $HP.dir().heap

returns a GuppyDoc object providing documentation for the heap
method. The method also takes a string argument specifying further
options. Currently the following are provided:

        'l'     Generate a listing of the synopsis lines.
        'L'     Generate a listing of the entire doc strings."""

        obj = self
        return self._root.guppy.etc.Help.dir(obj, opts)

    def _get_doc(self):
        """Overview documentation for top level Heapy object.
Provides a listing of the available attributes.
Accessing the attribute name on the doc objects gives further info, eg:

    >>> hp.doc.heap

gives doc for the heap method when hp is the top level Heapy object.

References may be embedded in the documentations. To access a
reference, opening up a web browser with the doc for it one can do eg:

    >>> hp.doc.heap[1]

The reference number 0 is special. If it is provided, it is the
reference to the html doc for the described object itself. So to see
in the web browser the doc for the heap method one can do:

    >>> hp.doc.heap[0]

References
    [0] heapy_Use.html#heapykinds.Use.doc"""

        return self._root.guppy.etc.Help.dir(self,
                                             header="""\
Top level interface to Heapy. Available attributes:""",
                                             footer="""\
Use eg: %sdoc.<attribute> for info on <attribute>.""" % self.reprefix)

    def _check_tracemalloc(self):
        if self.sys.version_info < (3, 8):
            self.warnings.warn(
                "Python 3.7 and below tracemalloc may not record accurate "
                "producer trace. See https://bugs.python.org/issue35053")
        if not self.tracemalloc.is_tracing():
            raise RuntimeError(
                "Tracemalloc is not tracing. No producer profile available. "
                "See https://docs.python.org/3/library/tracemalloc.html")

    def heapg(self, rma=1):
        """ DEPRECATED """
        self.warnings.warn(
            "Method Use.heapg is depreciated, it doesn't work well. Use heapu instead.")
        h = self.View.heapg(rma)
        h -= self.relheapg
        return h

    def heapu(self, rma=1, abs=0, stat=1):
        """heapu() -> Stat

Finds the objects in the heap that remain after garbage collection but
are _not_ reachable from the root.  This can be used to find objects
in extension modules that remain in memory even though they are
gc-collectable and not reachable.

Returns an object containing a statistical summary of the objects
found - not the objects themselves. This is to avoid making the
objects reachable.

See also: setref[1]

References
    [0] heapy_Use.html#heapykinds.Use.heapu
    [1] heapy_Use.html#heapykinds.Use.setref"""

        h = self.View.heapu(rma)
        rel = 0
        if not abs and self.relheapu and isinstance(self.relheapu, type(h)):
            h -= self.relheapu
            rel = 1
        if stat:
            h = h.stat
            if not abs and self.relheapu and isinstance(self.relheapu, type(h)):
                h -= self.relheapu
                rel = 1

            h.firstheader = 'Data from unreachable objects'

            if rel:
                h.firstheader += ' relative to: %s' %\
                                 self.ctime(self.relheapu.timemade)
            h.firstheader += '.\n'

        return h

    def heap(self):
        """heap() -> IdentitySet[1]

Traverse the heap from a root to find all reachable and visible
objects. The objects that belong to a heapy instance are normally not
included. Return an IdentitySet with the objects found, which is
presented as a table partitioned according to a default equivalence
relation (Clodo [3]).

See also: setref[2]

References
    [0] heapy_Use.html#heapykinds.Use.heap
    [1] heapy_UniSet.html#heapykinds.IdentitySet
    [2] heapy_Use.html#heapykinds.Use.setref
    [3] heapy_Use.html#heapykinds.Use.Clodo"""

        h = self.View.heap()
        h |= self.gcobjs
        h -= self.relheap
        return h

    def load(self, fn, use_readline=0):
        """\
load(alt:[fn: loadablefilenamestring+ or
          fn: loadableiterableofstrings+]
     [use_readline = boolean+]) -> Stat

Load heapy-related data from a serialized form. Currently it handles
data generated by Stat.dump.

Arguments
    fn: loadablefilenamestring+
        A string argument is treated as a file name.
    fn: loadableiterableofstrings+
        An open file or an iterator will be iterated over enough
        to read one package of data, and another call to load
        will read the next package.
    use_readline = boolean+
        If true, the method will use .readline() instead of
        iteration, which may be necessary in case the input
        comes from a pipe since otherwise the Python runtime
        would try to read ahead a big block before returning the
        first package of data.
Returns
    one package of statistical data.

References
    [0] heapy_Use.html#heapykinds.Use.load"""

        if isinstance(fn, str):
            # We got a filename.
            # I want to read only what is being requested
            # so I can look quickly at some lines of a long table.
            # (There are seemingly easier ways to do this
            #  but this takes care of some tricky details.
            #  Keeping f open avoids it to be overwritten
            #  (at least by Stat.dump() and if OS=Linux)
            #  if data are written to a new file with the same name.)
            f = open(fn)

            def get_trows():
                pos = 0
                while 1:
                    f.seek(pos)
                    line = f.readline()
                    if not line:
                        break
                    pos = f.tell()
                    yield line
        elif hasattr(fn, '__iter__') and not hasattr(fn, '__next__'):
            # We got a sequence, that is not an iterator. Use it directly.
            def get_trows():
                return fn
        elif hasattr(fn, '__next__'):
            # We got an iterator or file object.
            # We 'have' to read all lines (at once)-
            # to update the read position -
            # to mimic 'pickle' semantics if several
            # objects are stored in the same file.
            # We can't use .__next__ always - (eg not on pipes)
            # it makes a big readahead (regardless of buffering setting).
            # But since .__next__() (typically) is much faster, we use it
            # per default unless use_readline is set.
            if use_readline:
                get_line = fn.readline
            else:
                get_line = fn.__next__

            trows = []
            line = get_line()
            if not line:
                raise StopIteration
            endline = '.end: %s' % line
            try:
                while line:
                    trows.append(line)
                    if line == endline:
                        break
                    line = get_line()
                else:
                    raise StopIteration
            except StopIteration:
                trows.append(endline)

            def get_trows():
                return trows
        else:
            raise TypeError(
                'Argument should be a string, file or an iterable yielding strings.')

        a = next(iter(get_trows()))
        if not a.startswith('.loader:'):
            raise ValueError(
                'Format error in %r: no initial .loader directive.' % fn)
        loader = a[a.index(':')+1:].strip()
        try:
            loader = getattr(self, loader)
        except AttributeError:
            raise ValueError(
                'Format error in %r: no such loader: %r.' % (fn, loader))
        return loader(get_trows)

    def loadall(self, f):
        ''' Generates all objects from an open file f or a file named f'''
        if isinstance(f, str):
            f = open(f)
        while True:
            yield self.load(f)

    def loadc(self, fn):
        f = open(fn, 'r', 1)
        while 1:
            print(self.load(f, use_readline=1))

    def dumph(self, fn):
        f = open(fn, 'w')
        import gc
        while 1:
            x = self.heap()
            x.stat.dump(f)
            f.flush()
            print(len(gc.get_objects()))

    def setref(self, reachable=None, unreachable=None):
        """setref()

Set a reference point for heap usage measurement.  This applies to
both the heap[1] and heapu[2] methods. The heap() method will only
show the objects allocated after the time setref was called. The
heapu() method, since it deals with summary data and not actual
objects, will show the difference of sizes and counts compared to when
setref was called.

References
    [0] heapy_Use.html#heapykinds.Use.setref
    [1] heapy_Use.html#heapykinds.Use.heap
    [2] heapy_Use.html#heapykinds.Use.heapu"""

        if reachable is None and unreachable is None:
            self.setrelheap()
            self.setrelheapu()
        else:
            if reachable is not None:
                self.setrelheap(reachable)
            if unreachable is not None:
                self.setrelheapu(unreachable)

    def setrelheap(self, reference=None):
        if reference is None:
            reference = self.View.heap()
        self.relheap = reference

    def setrelheapg(self, reference=None):
        self.warnings.warn(
            "Method Use.setrelheapg is depreciated, use setref instead.")
        if reference is None:
            self.relheapg = None
            reference = self.View.heapg()
        self.relheapg = reference

    def setrelheapu(self, reference=None, stat=1):
        if reference is None:
            self.relheapu = None
            reference = self.heapu(abs=True, stat=stat)
        if stat and not isinstance(reference, self.Stat):
            reference = reference.stat
        self.relheapu = reference

    def test(self, debug=False):
        """test([debug: bool+ = False])

Run the Heapy test suite.

Argument
    debug
        If True, the tests will be run in debug mode so the stack frame
        can be examined with pdb.pm() after the first exception."""

        self._parent.test.test_all.test_main(debug)

    _imports_ = (
        '_parent.Classifiers:Clodo',
        '_parent.Classifiers:Id',
        '_parent.Classifiers:Idset',
        '_parent.Classifiers:Module',
        '_parent.Classifiers:Prod',
        '_parent.Classifiers:Rcs',
        '_parent.Classifiers:Size',
        '_parent.Classifiers:Type',
        '_parent.Classifiers:Unity',
        '_parent.Classifiers:Via',
        '_parent.Classifiers:findex',
        '_parent.Classifiers:sonokind',
        '_parent.Classifiers:tc_adapt',
        '_parent.Classifiers:tc_repr',
        '_parent.Monitor:monitor',
        '_parent.Part:_load_stat',
        '_parent.Part:Stat',
        '_parent.Prof:pb',
        '_parent.UniSet:Anything',
        '_parent.UniSet:idset',
        '_parent.UniSet:iso',
        '_parent.UniSet:Nothing',
        '_parent.UniSet:union',
        '_parent.UniSet:uniset_from_setcastable',
        '_parent:View',
        '_parent.View:_hiding_tag_',
        '_root:sys',
        '_root:tracemalloc',
        '_root.time:ctime',
        '_root:warnings',
    )

    _doc_Anything = """Anything: Kind

A symbolic set that represents all possible Python objects.

References
    [0] heapy_Use.html#heapykinds.Use.Anything"""

    _doc_Clodo = """Clodo:EquivalenceRelation
Clodo(alt:[tc: typeexceptdict+ or dictof =
        typeoremptytuple+]) -> Kind

Equivalence relation by class or dict owner. It distinguishes between
objects based on their type just like the Type relation, and in
addition distinguishes between dicts depending on what class they are
'owned' by, i.e. occur in __dict__ attribute of.

When called it returns the equivalence class defined by the argument,

EITHER:
    tc: A positional argument, a type but not a dict, to
        create the corresponding equivalence class.
OR:
    dictof: A named argument, to create an equivalence class
        consisting of all dicts that are owned by objects of the type
        specified in the argument; or dicts with no owner if
        an empty tuple is given. XXX express this simpler&better...


References
    [0] heapy_Use.html#heapykinds.Use.Clodo"""

    _doc_Id = """Id:EquivalenceRelation
Id(address: objectaddress+) -> Kind)

This equivalence relation defines objects to be equivalent only if
they are identical, i.e. have the same address. When called it returns
the equivalence class defined by the argument:

    address: The memory address of an object.

References
    [0] heapy_Use.html#heapykinds.Use.Id"""

    _doc_Idset = """Id:EquivalenceRelation
Idset(node: Anything+) -> IdentitySet

This equivalence relation defines objects to be equivalent only if
they are identical, i.e. have the same address. When called it returns
the equivalence class defined by the argument:

    node: Anything+
        Any object is a valid argument.

Note
    This is mainly for special purpose internal use. The Id
equivalence relation is more efficient when partitioning large
sets."""

    _doc_Module = """Module:EquivalenceRelation
x.Module( draw:[name = modulename+ , at = moduleaddress+]) -> Kind

This equivalence relation defines objects to be equivalent if they are
the same module, or if none of them is a module.  Partitioning a set
of objects using this equivalence relation will therefore result in
one singleton set for each module and one set containing all other
objects.

Calling the Module equivalence relation creates a Kind containing the
module given in the keyword argument(s). Either the name, address or
both may be specified. If no argument is specified the equivalence
class is that of non-module objects.

References
    [0] heapy_Use.html#heapykinds.Use.Module"""

    _doc_Nothing = """Nothing: IdentitySet

The empty set.

References
    [0] heapy_Use.html#heapykinds.Use.Nothing"""

    _doc_Prod = """\
Prod: EquivalenceRelation
Prod() -> KindOfProdFamily[1]
Prod(path_prefix: string+) -> Kind
Prod(scope: Anything+) -> Kind
Prod(seq: [filename: string+, alt: positive+]) -> KindOfProdFamily[1]

In this equivalence relation, objects are classified by the producer,
the line in which the object was allocated.

In zero-argument form, the equivalence relation creates a Kind matching
objects whose producer is unknown.

In one-argument form, the equivalence relation constructor creates an
alternative equivalence relation and has two possible types of arguments:

    path_prefix: string+
        A path prefix filter on the file name
    scope: Anything+
        Anything that can be used by inspect.getsourcelines()

In the two-argument form, the equivalence relation creates a Kind matching
objects whose producer matches exactly the given arguments:

    filename: string+
        File name of the producer
    lineno: positive+
        Line number of producer

With None arguments, the default equivalence relation will never match, but
alternate equivalence relations may, as described in KindOfProdFamily[1].

References
    [0] heapy_Use.html#heapykinds.Use.Prod
    [1] heapy_UniSet.html#heapykinds.KindOfProdFamily"""

    _doc_Rcs = """Rcs: EquivalenceRelation
Rcs ( 0..*: alt:[kind: Kind+ or sok: SetOfKind+]) -> KindOfRetClaSetFamily

(Referrer classification set.)

In this equivalence relation, objects are classified by classifying
their referrers, using the Clodo equivalence relation. These
classifications are collected in a set, representing the
classification of the object.

Calling Rcs creates an equivalence class from specified set of
referrer classifications. The arguments specify a set of Kind objects,
each of which representing an equivalence class of Clodo.

    kind: Kind+
        This adds a single Kind to the set of Kinds of referrers.
    sok: SetOfKind+
        This adds each Kind in the sok argument to the total set of
        Kinds of referrers.
References
    [0] heapy_Use.html#heapykinds.Use.Rcs"""

    _doc_Size = """\
Size: EquivalenceRelation
Size(size: notnegative+) -> KindOfSizeFamily[1])

In this equivalence relation, objects are classified by memory size,
so each equivalence class represents a particular size of object.

References
    [0] heapy_Use.html#heapykinds.Use.Size
    [1] heapy_UniSet.html#heapykinds.KindOfSizeFamily"""

    _doc_Type = """Type: EquivalenceRelation
Type(type: type+) -> KindOfTypeFamily[1]

In this equivalence relation, objects are classified by type so each
equivalence class represents objects of a particular type.  Calling it
creates a Kind representing the type specified in the argument:

    type: type+
        A Python type object or a representation of it.

References
    [0] heapy_Use.html#heapykinds.Use.Type
    [1] heapy_UniSet.html#heapykinds.KindOfTypeFamily"""

    _doc_Unity = """Unity: EquivalenceRelation
Unity() -> Kind[1]

In this equivalence relation, all objects are considered equivalent.
There is only one equivalence class, that is, Anything[2].

References
    [0] heapy_Use.html#heapykinds.Use.Unity
    [1] heapy_UniSet.html#heapykinds.Kind
    [2] heapy_Use.html#heapykinds.Use.Anything"""

    _doc_Via = """Via: EquivalenceRelation
Via( 0..*:rel: relationname+) -> KindOfInViaFamily[1]

In this equivalence relation, objects are classified by how they are
referred from their referrers, so each equivalence class represents
objects that have a particular set of relations to their referrers.
Calling it creates a Kind representing the set of referrers specified
by the argument:

    rel: relationname+
        Each argument specifies one referrer relation. The arguments
        should be strings and can be of any of the following forms.

[expression]
    Indexing of a dict, list, tuple (etc).  The expression must be a
    Python expression that can be evaluated in a local
    environment. The environment will contain the builtins and a name
    'hp' that is bound to the current Use instance.

.attribute
    Getting an attribute from a builtin type or a slot of a slotted
    type. (I.E. not an attribute that is in a dict of an object.)

.f_locals["name"]
     A local variable of a frame.

.f_locals ["name"]
    A variable in a CELL of a frame. Note the space between f_locals and
    [. This is to distinguish it from ordinary locals, and still use a
    syntax that could be used to access those variables directly from
    Python.

.keys()[integer]
    A key in a dictionary, at the indicated place in its keys().

References
    [0] heapy_Use.html#heapykinds.Use.Via
    [1] heapy_UniSet.html#heapykinds.KindOfInViaFamily"""

    _doc_findex = """
findex( 0..*:kind: Kind+) -> (
    Subkind of: EquivalenceRelation[1]
    callable: (index: notnegative+)
        Calling the returned equivalence relation creates an
        equivalence class.
        Argument
            index: notnegative+
                The position of the matching kind in the sequence of
                kinds. The first one has index 0. Specifying the
                length of the sequence means that the equivalence
                class returned is the one where none of the kinds in
                the sequence matched.

    )

Create an equivalence relation based on a sequence of kinds. The
name is a combination of find and index. The classification of
each objects is done as follows:

For each kind in the sequence, check whether the object is an
element of that kind. If it is, the classification is the index
of that kind in the sequence. If the end of the sequence is
reached, the classification is the length of the sequence.
Argument
    kind: Kind+
        Each argument specifies the kind in that position in the
        sequence.
Bugs
    Though the Kind objects representing the equivalence classes
    work with set operations such as intersection and union, the
    tests such as subset and equality do not generally give the
    expected result.
References
    [0] heapy_Use.html#heapykinds.Use.findex
    [1] heapy_UniSet.html#heapykinds.EquivalenceRelation"""

    _doc_idset = """idset(nodes: iterable+) -> IdentitySet[1]

Create a set of objects based on identity.

Argument
    nodes: iterable+
        The argument must be an iterable and may yield any kind
        of objects.
Note
    This method is the same as iso except for the argument.
References
    [0] heapy_Use.html#heapykinds.Use.idset
    [1] heapy_UniSet.html#heapykinds.IdentitySet"""

    _doc_iso = """iso( 0..*:node: Any+) -> IdentitySet[1]

Create a set of objects based on identity.

Argument
    node: Any+
        Any kind of objects are valid arguments.
Note
    This method is the same as idset[2] except for the argument.
References
    [0] heapy_Use.html#heapykinds.Use.iso
    [1] heapy_UniSet.html#heapykinds.IdentitySet
    [2] heapy_Use.html#heapykinds.Use.idset"""

    _doc_sokind = """
"""



class Format(object):
    __slots__ = 'impl', 'mod'

    def __init__(self, impl):
        self.impl = impl
        self.mod = impl.mod

    def get_formatted_row(self, row):
        fr = self.get_stat_data(row)
        rows = []
        rs = row.name.split('\n')
        subsequent_indent = len(fr)*' '
        rows.extend(self.mod.wrap(
            fr+rs[0],
            width=self.mod.line_length,
            subsequent_indent=subsequent_indent))

        for r in rs[1:]:
            rows.extend(self.mod.wrap(
                r,
                width=self.mod.line_length,
                initial_indent=subsequent_indent,
                subsequent_indent=subsequent_indent))
        return '\n'.join(rows)

    def load_statrow_csk(self, r):
        impl = self.impl
        count, size, kind = r.split(' ', 2)
        count = int(count)
        size = int(size)
        impl.cum_size += size
        return StatRow(count, size, kind, impl.cur_index, impl.cum_size)

    def load_statrow_sk(self, r):
        impl = self.impl
        size, kind = r.split(' ', 1)
        size = int(size)
        impl.cum_size += size
        return StatRow(1, size, kind, impl.cur_index, impl.cum_size)

    def _oh_get_num_lines(self):
        return self.impl.numrows

    def _oh_get_label(self):
        return self.get_label()

    def _oh_get_row_header(self):
        impl = self.impl
        if not (impl.count or impl.size):
            return ''
        sh = self.get_stat_header()
        return self.mod.fill(
            sh + self.impl.kindheader,
            width=self.mod.line_length,
            subsequent_indent=' '*len(sh))

    def _oh_get_more_msg(self, start_lineno, end_lineno):
        nummore = self.impl.numrows-(end_lineno+1)
        return "<%d more rows. Type e.g. '_.more' to view.>" % nummore

    def _oh_get_line_iter(self):
        for numrows, row in enumerate(self.impl.get_rows()):
            yield self.get_formatted_row(row)


class SetFormat(Format):
    __slots__ = ()

    def get_label(self):
        impl = self.impl
        if impl.count != 1:
            s = 's'
        else:
            s = ''
        return 'Partition of a set of %d object%s. Total size = %d bytes.' % (
            impl.count, s, impl.size)

    def get_rowdata(self, row):
        return '%d %d %s' % (row.count, row.size, row.name)

    def get_stat_header(self):
        return (
            ' Index  Count   %     Size   % Cumulative  % ')

    def get_stat_data(self, row):
        format = '%6d %6d %3d %8d %3d %9d %3d '
        impl = self.impl
        fr = format % (
            row.index,
            row.count, int('%.0f' % (row.count * 100.0/impl.count)),
            row.size, int('%.0f' % (row.size * 100.0/impl.size)),
            row.cumulsize, int('%.0f' % (row.cumulsize * 100.0/impl.size)),
        )
        return fr

    def load_statrow(self, r):
        return self.load_statrow_csk(r)


class IdFormat(Format):
    __slots__ = ()

    def get_label(self):
        impl = self.impl
        if impl.count != 1:
            s = 's'
        else:
            s = ''
        return (
            'Set of %d %s object%s. Total size = %d bytes.' % (
                impl.count, impl.kindname, s, impl.size))
        return part

    def get_rowdata(self, row):
        return '%d %s' % (row.size, row.name)

    def get_stat_header(self):
        return (
            ' Index     Size   %   Cumulative  %   ')

    def get_stat_data(self, row):
        impl = self.impl
        format = '%6d %8d %5.1f %9d %5.1f '
        fr = format % (
            row.index,
            row.size, (row.size * 100.0/impl.size),
            row.cumulsize, row.cumulsize * 100.0/impl.size,
        )
        return fr

    def load_statrow(self, r):
        return self.load_statrow_sk(r)


class DiffFormat(Format):
    __slots__ = ()

    def _percent_of_b(self, size):
        if self.impl.b_size != 0:
            return '%9.3g' % (size*100.0/self.impl.b_size,)
        else:
            return '   (n.a.)'

    def get_label(self):
        impl = self.impl
        x = (
            'Summary of difference operation (A-B).\n' +
            '        Count     Size\n' +
            '  A    %6d %8d\n' % (impl.count+impl.b_count, impl.size+impl.b_size) +
            '  B    %6d %8d\n' % (impl.b_count, impl.b_size) +
            '  A-B  %6d %8d  = %s %% of B\n' % (impl.count, impl.size, self._percent_of_b(impl.size)))

        if impl.count or impl.size:
            x += '\nDifferences by kind, largest absolute size diffs first.'
        return x

    def get_rowdata(self, row):
        return '%d %d %s' % (row.count, row.size, row.name)

    def get_stat_header(self):
        return (
            ' Index  Count     Size  Cumulative  % of B ')

    def get_stat_data(self, row):
        impl = self.impl
        format = '%6d %6d %8d %9d %s '
        fr = format % (
            row.index,
            row.count,
            row.size,
            row.cumulsize,
            self._percent_of_b(row.cumulsize),
        )
        return fr

    def load_statrow(self, r):
        return self.load_statrow_csk(r)


class StatRow(object):
    __slots__ = 'count', 'size', 'name', 'index', 'cumulsize'

    def __init__(self, count, size, name, index=None, cumulsize=None):
        self.count = count
        self.size = size
        self.name = name
        self.index = index
        self.cumulsize = cumulsize


class PartRow(StatRow):
    __slots__ = 'set', 'kind'

    def __init__(self, count, size, name, index, cumulsize, set, kind):
        self.count = count
        self.size = size
        self.name = name
        self.index = index
        self.cumulsize = cumulsize
        self.set = set
        self.kind = kind


class Stat:
    def __init__(self, mod, get_trows, firstheader=''):
        self.mod = mod
        self._hiding_tag_ = mod._hiding_tag_
        self.get_trows = get_trows
        self.firstheader = firstheader

        self.it = iter(get_trows())

        self.cur_index = 0
        self.cum_size = 0
        self.rows = []

        r = self.get_next()
        while r and not r.startswith('.r:'):
            name = r[1:r.index(':')]
            value = r[r.index(':')+1:].strip()
            try:
                value = int(value)
            except ValueError:
                pass
            setattr(self, name, value)
            r = self.get_next()

        self.format_name = self.format
        self.format_class = getattr(self.mod, self.format)
        self.format = self.format_class(self)

        self.timemade = float(self.timemade)

        self.mod.OutputHandling.setup_printing(self, self.format)

    def __getitem__(self, idx):
        if isinstance(idx, int):
            if idx < 0:
                idx = self.numrows + idx
            if not (0 <= idx < self.numrows):
                raise IndexError('Stat index out of range.')
            rows = [self.get_row(idx)]
        elif isinstance(idx, slice):
            start, stop, step = idx.indices(self.numrows)
            rows = [self.get_row(idx) for idx in range(start, stop, step)]
        else:
            raise IndexError('Stat indices must be integers or slices.')

        count = 0
        size = 0
        for r in rows:
            count += r.count
            size += r.size

        trows = [
            '.loader: _load_stat',
            '.format: %s' % self.format_name,
            '.timemade: %f' % self.timemade,
            '.count: %d' % count,
            '.size: %d' % size,
            '.kindheader: %s' % self.kindheader,
            '.kindname: %s' % self.kindname,
            '.numrows: %d' % len(rows),
        ]
        if getattr(self, 'b_count', None) is not None:
            trows.append('.b_count: %d' % self.b_count)
            trows.append('.b_size: %d' % self.b_size)
        for r in rows:
            trows.append('.r: %s' % self.format.get_rowdata(r))
        return self.mod.load(trows)

    def __len__(self):
        return self.numrows

    def __sub__(self, other):
        if not isinstance(other, Stat):
            raise TypeError(
                'Can only take difference with other Stat instance.')
        if self.kindheader != other.kindheader:
            raise ValueError('Mismatching table kind header, %r vs %r.' % (
                self.kindheader, other.kindheader))
        rows = []
        otab = {}
        stab = {}
        for r in other.get_rows():
            o = otab.get(r.name)
            if o:
                otab[r.name] = StatRow(
                    r.count+o.count, r.size+o.size, r.name, o.index,  None)
            else:
                otab[r.name] = r
        for r in self.get_rows():
            o = otab.get(r.name)
            if o:
                del otab[r.name]
                count = r.count - o.count
                size = r.size - o.size
            else:
                count = r.count
                size = r.size
            if count == 0 and size == 0:
                continue
            sr = stab.get(r.name)
            if sr:
                sr.count += count
                sr.size += size
            else:
                sr = StatRow(count, size, r.name)
                stab[sr.name] = sr
                rows.append(sr)
        rs = list(otab.values())
        rs.sort(key=lambda x: x.index)  # Preserve orig. order
        for r in rs:
            sr = StatRow(-r.count, -r.size, r.name)
            assert sr.name not in stab
            rows.append(sr)
        rows.sort(key=lambda x: abs(x.size), reverse=True)
        cumulcount = 0
        cumulsize = 0
        for r in rows:
            cumulcount += r.count
            cumulsize += r.size
            r.cumulsize = cumulsize
        trows = [
            '.loader: _load_stat',
            '.format: DiffFormat',
            '.timemade: %f' % self.mod.time.time(),
            '.b_count: %d' % other.count,
            '.b_size: %d' % other.size,
            '.count: %d' % cumulcount,
            '.size: %d' % cumulsize,
            '.kindheader: %s' % self.kindheader,
            '.kindname: %s' % self.kindname,
            '.numrows: %d' % len(rows),
        ]
        for r in rows:
            trows.append('.r: %d %d %s' % (r.count, r.size, r.name))
        return self.mod.load(trows)

    def dump(self, fn, mode='a'):
        if not hasattr(fn, 'write'):
            f = open(fn, mode)
        else:
            f = fn
        try:
            for r in self.get_trows():
                if not r[-1:] == '\n':
                    r += '\n'
                f.write(r)
            end = '.end: .loader: %s\n' % self.loader
            if r != end:
                f.write(end)
        finally:
            if f is not fn:
                f.close()

    def get_next(self):
        try:
            r = next(self.it)
        except StopIteration:
            r = None
        else:
            r = r.rstrip('\n')
        self.last = r
        return r

    def get_row(self, idx):
        while idx >= len(self.rows):
            self.parse_next_row()
        return self.rows[idx]

    def get_rows(self, idx=0):
        while idx < self.numrows:
            try:
                row = self.get_row(idx)
            except IndexError:
                return
            else:
                yield row
            idx += 1

    def get_rows_of_kinds(self, kinds):
        # Return the rows with names in sequence kinds of unique names
        # in that order. None if no such kind.

        kindtab = {}
        N = len(kinds)
        res = [None] * len(kinds)
        for i, kind in enumerate(kinds):
            kindtab[kind] = i
        assert len(kindtab) == N

        n = 0
        for row in self.get_rows():
            idx = kindtab.get(row.name)
            if idx is not None:
                res[idx] = row
            n += 1
            if n >= N:
                break
        return res

    def get_rows_n_and_other(self, N, sortby='Size'):
        # Get N rows, the largest first
        # mix in an '<Other>' row at a sorted position
        # Size is either size if sortby = 'Size',
        #    or count if sortby = 'Count'.
        # Returns a NEW LIST (caller may modify/sort it)

        if sortby not in ('Size', 'Count'):
            raise ValueError("Argument 'sortby' must be 'Size' or 'Count'.")

        # Rows are already sorted by Size, largest first.
        # If they want by Count, we need to resort them.

        rows = self.get_rows()

        if sortby == 'Count':
            rows = list(rows)
            rows.sort(key=lambda x: x.count, reverse=True)

        retrows = []
        cumulcount = 0
        cumulsize = 0

        for (i, r) in enumerate(rows):
            if i >= N:
                othercount = self.count - cumulcount
                othersize = self.size - cumulsize
                other = StatRow(othercount,
                                othersize,
                                '<Other>')

                if sortby == 'Size':
                    for (i, r) in enumerate(retrows):
                        if r.size < othersize:
                            retrows[i:i] = [other]
                            break
                    else:
                        retrows.append(other)
                elif sortby == 'Count':
                    for (i, r) in enumerate(retrows):
                        if r.count < othercount:
                            retrows[i:i] = [other]
                            break
                    else:
                        retrows.append(other)
                else:
                    assert 0
                break

            cumulcount += r.count
            cumulsize += r.size
            retrows.append(r)
        else:
            assert cumulcount == self.count
            assert cumulsize == self.size

        return retrows

    def parse_next_row(self):
        r = self.last
        if not r:
            raise IndexError('Row index out of range.')
        if r.startswith('.r: '):
            r = r[4:]
            sr = self.format.load_statrow(r)

            self.cur_index += 1
            self.rows.append(sr)
            self.get_next()
            return

        elif r.startswith('.end'):
            raise IndexError('Row index out of range.')
        else:
            raise SyntaxError


class Partition:
    def __init__(self, mod, set, er):
        self.mod = mod
        self.set = set
        self.er = er
        self._hiding_tag_ = mod._hiding_tag_
        self.timemade = mod.time.time()

    def __iter__(self):
        # The default iteration is over the sets
        # To iterate over rows (if more info is needed), get_rows() is available.
        return self.get_sets()

    def get_rows(self, rowindex=None):
        # Iterator over rows
        if rowindex is None:
            rowindex = 0
        while 1:
            try:
                row = self.get_row(rowindex)
            except IndexError:
                return
            else:
                yield row
            rowindex += 1

    def get_set(self, index):
        if isinstance(index, slice):
            start, stop, step = index.indices(self.numrows)
            ns = self.get_nodeset(start, stop, step)
            return self.mod.idset(ns, er=self.er)
        else:
            if index < 0:
                index += self.numrows
            return self.get_rowset(index)

    def get_sets(self, index=None):
        for idx in range(self.numrows):
            yield self.get_rowset(idx)

    def get_stat(self):
        # Avoid any references into the set!
        trows = list(self.get_trows())

        def get_trows():
            return trows
        return self.mod._load_stat(get_trows)

    def get_trows(self):
        yield '.loader: _load_stat'
        yield '.format: %s' % self.format.__class__.__name__
        yield '.timemade: %f' % self.timemade
        yield '.count: %d' % self.count
        yield '.size: %d' % self.size
        yield '.kindname: %s' % self.kindname
        yield '.kindheader: %s' % self.kindheader
        yield '.numrows: %d' % self.numrows
        for row in self.get_rows():
            yield '.r: %s' % self.format.get_rowdata(row)

    def init_format(self, FormatClass):
        self.format = FormatClass(self)

        self.mod.OutputHandling.setup_printing(self, self.format)


class IdentityPartitionCluster(object):
    # Contains objects of same size.
    # to speed up management of identity partition
    # - since otherwise we'd have to sort all the objects,
    #   on their string representation in worst case.
    __slots__ = 'objects', 'locount', 'hicount', 'losize', 'obsize', 'issorted'

    def __init__(self, objects, locount, count, losize, obsize):
        self.objects = objects          # tuple of objects in this segment
        self.locount = locount          # count BEFORE objects in this cluster
        self.hicount = locount+count    # count AFTER these objects
        self.losize = losize            # size BEFORE  objects in this cluster
        self.obsize = obsize            # size of EACH object in this segment
        self.issorted = False           # indicates if .objects is sorted


class IdentityPartition(Partition):
    def __init__(self, mod, set, er):
        Partition.__init__(self, mod, set, er)

        clusters = []
        sizeclasses = mod.Size.classifier.partition_cli(set.nodes)
        sizeclasses.sort()
        sizeclasses.reverse()
        totcount = 0
        totsize = 0
        for size, v in sizeclasses:
            count = len(v)
            clusters.append(IdentityPartitionCluster(
                self.mod.observation_list(v), totcount, count, totsize, size))
            totsize += size * count
            totcount += count
        assert totcount == set.count

        self.cluidx = 0
        self.clusters = clusters
        self.count = totcount
        self.kind = kind = set.byclodo.kind
        self.kindheader = kind.fam.c_get_idpart_header(kind)
        self.kindname = kind.fam.c_get_idpart_label(kind)
        self.numrows = totcount
        self.render = kind.fam.c_get_idpart_render(kind)
        self.size = totsize
        self.sortrender = kind.fam.c_get_idpart_sortrender(kind)

        self.init_format(IdFormat)

    def get_nodeset(self, start, stop, step):
        return self.get_nodeset_cluster(start, stop, step)[0]

    def get_nodeset_cluster(self, start, stop, step):
        if step <= 0:
            raise ValueError('Step must be positive.')
        ns = self.mod.mutnodeset()
        if start >= stop:
            return (ns, None)
        clusters = self.clusters
        lo = 0
        hi = len(clusters)
        cluidx = self.cluidx

        while lo < hi:
            clu = clusters[cluidx]
            if clu.locount <= start:
                if start < clu.hicount:
                    break
                else:
                    lo = cluidx + 1
            else:
                hi = cluidx
            cluidx = (lo + hi) // 2
        else:
            return (ns, None)
        clu_to_return = clu

        while 1:
            objects = clu.objects
            if start != clu.locount or stop < clu.hicount or step != 1:
                if not clu.issorted:
                    sortrender = self.sortrender
                    if sortrender == 'IDENTITY':
                        ks = objects
                    else:
                        ks = [sortrender(x) for x in objects]
                    ks = [(kind, i) for i, kind in enumerate(ks)]
                    ks.sort()
                    clu.objects = objects = self.mod.observation_list(
                        [objects[i] for (kind, i) in ks])
                    clu.issorted = True
                objects = objects[start-clu.locount:stop-clu.locount:step]

            ns |= objects
            self.cluidx = cluidx  # memo till next call
            start += len(objects)*step
            if start >= stop:
                break
            for cluidx in range(cluidx + 1, len(clusters)):
                clu = clusters[cluidx]
                if clu.locount <= start < clu.hicount:
                    break
            else:
                break
        return (ns, clu_to_return)

    def get_row(self, rowidx):
        ns, clu = self.get_nodeset_cluster(rowidx, rowidx+1, 1)
        if not ns:
            raise IndexError('Partition index out of range.')
        vi = self.mod.idset(ns, er=self.er)
        row = PartRow(1, clu.obsize, self.render(vi.theone),
                      rowidx, (rowidx+1-clu.locount)*clu.obsize + clu.losize,
                      vi, vi.kind)
        return row

    def get_rowset(self, rowidx):
        ns = self.get_nodeset(rowidx, rowidx+1, 1)
        if not ns:
            raise IndexError('Partition index out of range.')
        return self.mod.idset(ns, er=self.er)


class SetPartition(Partition):
    def __init__(self, mod, set, er):
        Partition.__init__(self, mod, set, er)

        classifier = er.classifier
        tosort = [(-part.size, classifier.get_tabrendering(kind, ''), kind, part)
                  for (kind, part) in classifier.partition(set.nodes)]
        tosort.sort()
        cumulsize = 0
        rows = []
        for (minusize, name, kind, part) in tosort:
            size = -minusize
            cumulsize += size
            rows.append(PartRow(
                part.count, size, name,
                len(rows), cumulsize,
                part, kind))

        self.count = set.count
        self.kindheader = classifier.get_tabheader('')
        self.kindname = ''
        self.numrows = len(rows)
        self.rows = rows
        self.size = cumulsize

        self.init_format(SetFormat)

    def get_nodeset(self, start, stop, step):
        if step <= 0:
            raise ValueError('Step must be positive.')
        ns = self.mod.mutnodeset()
        while start < stop:
            ns |= self.rows[start].set.nodes
            start += step
        return ns

    def get_row(self, idx):
        try:
            return self.rows[idx]
        except IndexError:
            raise IndexError('Partition index out of range.')

    def get_rowset(self, idx):
        row = self.get_row(idx)
        set = row.set
        try:
            set._partition
        except AttributeError:
            set._partition = self.mod._fast_partition(self, row)
        return set


class FastPartition(SetPartition):
    def __init__(self, mod, parent, row):
        Partition.__init__(self, mod, parent.set, parent.er)

        self.count = row.count
        self.kindheader = parent.kindheader
        self.kindname = ''
        self.numrows = 1
        self.rows = [row]
        self.size = row.size

        self.init_format(SetFormat)


class _GLUECLAMP_:
    _preload_ = ('_hiding_tag_',)
    _chgable_ = ('line_length', 'backup_suffix')
    _imports_ = (
        '_parent.ImpSet:mutnodeset',
        '_parent:OutputHandling',
        '_parent.Use:Id',
        '_parent.Use:Size',
        '_parent.Use:idset',
        '_parent.Use:load',
        '_parent.View:_hiding_tag_',
        '_parent.View:observation_list',
        '_root.os:rename',
        '_root.textwrap:fill',
        '_root.textwrap:wrap',
        '_root.textwrap:wrap',
        '_root:time',
    )

    # 'Config'

    line_length = 100
    backup_suffix = '.old'

    # Factory method

    def partition(self, set, er):
        if er.classifier is self.Id.classifier:
            return IdentityPartition(self, set, er)
        else:
            return SetPartition(self, set, er)

    # Private - Use.load is intended to be used directly.
    def _load_stat(self, get_trows):
        return Stat(self, get_trows)

    def _fast_partition(self, parent, row):
        return FastPartition(self, parent, row)



from io import StringIO


def reptable(tb):
    if not tb:
        return 0, []
    maxlens = [0]*len(tb[0])
    for r in tb:
        if r == '-':
            continue
        for i, e in enumerate(r):
            maxlens[i] = max(maxlens[i], len(str(e))+1)

    sumlens = len(maxlens)
    for s in maxlens:
        sumlens += s

    out = []
    for r in tb:
        if r == '-':
            out.append('-'*min(sumlens, 75))
        else:
            so = ''
            for i, e in enumerate(r):
                s = str(e)
                if s.startswith('!>'):
                    s = s[2:]
                    fillright = 1
                elif s.isdigit():
                    fillright = 1
                else:
                    fillright = 0
                ml = maxlens[i]-1
                fill = ' '*(ml - len(s))
                if fillright:
                    s = fill + s
                else:
                    s = s + fill
                so += s + ' '
            out.append(so)
    return maxlens, out


def ptable(tb, f=None):
    if f is None:
        import sys
        f = sys.stdout
    _, lines = reptable(tb)
    for line in lines:
        line = line.rstrip()
        print(line, file=f)


def strtable(tb):
    f = StringIO()
    ptable(tb, f)
    return f.getvalue()


def str2int(s, msg='Hexadecimal literal in the form [-]0x... expected'):
    # xxx clumsy -- there should be a builtin function for this !
    if s.startswith('-'):
        sign = -1
        s = s[1:]
    else:
        sign = 1
    if not s.startswith('0x'):
        raise ValueError(msg)
    s = s[2:]
    if s.endswith('l') or s.endswith('L'):
        s = s[:-1]
    return int(s, 16) * sign



# Test the gsl subpackage
# Ideally this should be a top level test.


def test_main(debug=0):
    from guppy import Root
    gsl = Root().guppy.gsl
    gsl.Document._test_main_()
    gsl.DottedTree.test_main()
    gsl.FileIO.set_test_mode()
    gsl.Filer._test_main_()
    gsl.Gsml._test_main_()
    gsl.Main._test_main_()
    gsl.SpecNodes.test_main()
    # gsl.Text.test()


if __name__ == "__main__":
    test_main()
